@@ -27,6 +27,7 @@ import stat
 import subprocess
 import sys
 import tempfile
+import warnings
 
 import apport.fileutils
 from apport.packaging_impl import impl as packaging
@@ -167,56 +168,26 @@ def attach_conffiles(report, package, conffiles=None, ui=None):
         report[f"mtime.conffile.{path_to_key(path)}"] = mtime.isoformat()
 
 
+# pylint: disable-next=unused-argument
 def attach_upstart_overrides(report, package):
     """Attach information about any Upstart override files."""
-    try:
-        files = packaging.get_files(package)
-    except ValueError:
-        return
-
-    for file in files:
-        if os.path.exists(file) and file.startswith("/etc/init/"):
-            override = file.replace(".conf", ".override")
-            key = f"upstart.{override.replace('/etc/init/', '')}"
-            attach_file_if_exists(report, override, key)
+    warnings.warn(
+        "apport.hookutils.attach_upstart_overrides() is obsolete."
+        " Upstart is dead. Please drop this call.",
+        PendingDeprecationWarning,
+        stacklevel=2,
+    )
 
 
+# pylint: disable-next=unused-argument
 def attach_upstart_logs(report, package):
     """Attach information about a package's session upstart logs."""
-    try:
-        files = packaging.get_files(package)
-    except ValueError:
-        return
-
-    for f in files:
-        if not os.path.exists(f):
-            continue
-        if f.startswith("/usr/share/upstart/sessions/"):
-            log = os.path.basename(f).replace(".conf", ".log")
-            key = f"upstart.{log}"
-            try:
-                log = os.path.join(os.environ["XDG_CACHE_HOME"], "upstart", log)
-            except KeyError:
-                try:
-                    log = os.path.join(os.environ["HOME"], ".cache", "upstart", log)
-                except KeyError:
-                    continue
-
-            attach_file_if_exists(report, log, key)
-
-        if f.startswith("/usr/share/applications/") and f.endswith(".desktop"):
-            desktopname = os.path.splitext(os.path.basename(f))[0]
-            key = f"upstart.application.{desktopname}"
-            log = f"application-{desktopname}.log"
-            try:
-                log = os.path.join(os.environ["XDG_CACHE_HOME"], "upstart", log)
-            except KeyError:
-                try:
-                    log = os.path.join(os.environ["HOME"], ".cache", "upstart", log)
-                except KeyError:
-                    continue
-
-            attach_file_if_exists(report, log, key)
+    warnings.warn(
+        "apport.hookutils.attach_upstart_logs() is obsolete."
+        " Upstart is dead. Please drop this call.",
+        PendingDeprecationWarning,
+        stacklevel=2,
+    )
 
 
 def attach_dmesg(report):
@@ -233,13 +204,13 @@ def attach_dmi(report):
     dmi_dir = "/sys/class/dmi/id"
     if os.path.isdir(dmi_dir):
         for f in os.listdir(dmi_dir):
+            if f in {"subsystem", "uevent"}:
+                continue
             p = os.path.realpath(f"{dmi_dir}/{f}")
             st = os.stat(p)
             # ignore the root-only ones, since they have serial numbers
             if not stat.S_ISREG(st.st_mode) or (st.st_mode & 4 == 0):
                 continue
-            if f in ("subsystem", "uevent"):
-                continue
 
             try:
                 value = read_file(p)
@@ -248,6 +219,12 @@ def attach_dmi(report):
             if value:
                 report[f"dmi.{f.replace('_', '.')}"] = value
 
+    # Use the hardware information to create a machine type.
+    if "dmi.sys.vendor" in report and "dmi.product.name" in report:
+        report["MachineType"] = (
+            f"{report['dmi.sys.vendor']} {report['dmi.product.name']}"
+        )
+
 
 def attach_hardware(report):
     """Attach a standard set of hardware-related data to the report, including:
@@ -295,12 +272,6 @@ def attach_hardware(report):
 
     attach_dmi(report)
 
-    # Use the hardware information to create a machine type.
-    if "dmi.sys.vendor" in report and "dmi.product.name" in report:
-        report[
-            "MachineType"
-        ] = "{report['dmi.sys.vendor']} {report['dmi.product.name']}"
-
     if command_available("prtconf"):
         report["Prtconf"] = command_output(["prtconf"])
 
@@ -762,10 +733,10 @@ def attach_gsettings_schema(report, schema):
                 continue  # invalid line
 
             if value != defaults.get(schema_name, {}).get(key, ""):
-                if schema_name == b"org.gnome.shell" and key in [
+                if schema_name == b"org.gnome.shell" and key in {
                     b"command-history",
                     b"favorite-apps",
-                ]:
+                }:
                     value = "redacted by apport"
                 cur_value += f"{schema_name} {key} {value}\n"
 
@@ -1104,9 +1075,11 @@ def attach_default_grub(report, key=None):
     if os.path.exists(path):
         with open(path, "r", encoding="utf-8") as f:
             filtered = [
-                line
-                if not line.startswith("password")
-                else "### PASSWORD LINE REMOVED ###"
+                (
+                    line
+                    if not line.startswith("password")
+                    else "### PASSWORD LINE REMOVED ###"
+                )
                 for line in f.readlines()
             ]
             report[key] = "".join(filtered)
