@@ -39,12 +39,12 @@ import sys
 import threading
 import time
 import traceback
-import typing
 import urllib.error
 import webbrowser
 import zlib
+from collections.abc import Sequence
 from gettext import gettext as _
-from typing import Any, Optional
+from typing import Any
 
 import apport.crashdb
 import apport.fileutils
@@ -53,7 +53,7 @@ import apport.REThread
 from apport.packaging_impl import impl as packaging
 from apport.user_group import get_process_user_and_group
 
-__version__ = "2.27.0"
+__version__ = "2.28.1"
 
 
 symptom_script_dir = os.environ.get("APPORT_SYMPTOMS_DIR", "/usr/share/apport/symptoms")
@@ -69,7 +69,7 @@ def get_pid(report):
         return None
 
 
-def _get_env_int(key: str, default: Optional[int] = None) -> Optional[int]:
+def _get_env_int(key: str, default: int | None = None) -> int | None:
     """Get an environment variable as integer.
 
     Return None if it doesn't exist or failed to convert to integer.
@@ -81,7 +81,7 @@ def _get_env_int(key: str, default: Optional[int] = None) -> Optional[int]:
         return default
 
 
-def _get_newest_process_for_user(name: str, uid: int) -> Optional[int]:
+def _get_newest_process_for_user(name: str, uid: int) -> int | None:
     process = subprocess.run(
         ["pgrep", "-n", "-x", "-u", str(uid), name],
         capture_output=True,
@@ -116,7 +116,7 @@ def _get_users_environ(uid: int) -> dict[str, str]:
 
     return {
         key: gvfsd_env[key]
-        for key in ["DBUS_SESSION_BUS_ADDRESS", "XDG_DATA_DIRS"]
+        for key in ("DBUS_SESSION_BUS_ADDRESS", "XDG_DATA_DIRS")
         if key in gvfsd_env
     }
 
@@ -347,8 +347,8 @@ class UserInterface:
     def __init__(self, argv: list[str]):
         """Initialize program state and parse command line options."""
         self.gettext_domain = "apport"
-        self.report: Optional[apport.report.Report] = None
-        self.report_file: Optional[str] = None
+        self.report: apport.report.Report | None = None
+        self.report_file: str | None = None
         self.cur_package = None
         self.offer_restart = False
         self.specified_a_pkg = False
@@ -395,35 +395,13 @@ class UserInterface:
 
         if os.geteuid() == 0:
             reports = apport.fileutils.get_new_system_reports()
-            logind_session = None
         else:
             reports = apport.fileutils.get_new_reports()
-            proc_pid_fd = os.open(
-                f"/proc/{os.getpid()}", os.O_RDONLY | os.O_PATH | os.O_DIRECTORY
-            )
-            logind_session = apport.Report.get_logind_session(proc_pid_fd=proc_pid_fd)
 
         for f in reports:
             if not self.load_report(f):
                 continue
-
-            # Skip crashes that happened during logout, which are uninteresting
-            # and confusing to see at the next login. A crash happened and gets
-            # reported in the same session if the logind session paths agree
-            # and the session started before the report's "Date".
-            if (
-                logind_session
-                and "_LogindSession" in self.report
-                and "Date" in self.report
-            ):
-                # report.get_timestamp() can return None
-                if not self.report.get_timestamp():
-                    continue
-                if (
-                    logind_session[0] != self.report["_LogindSession"]
-                    or logind_session[1] > self.report.get_timestamp()
-                ):
-                    continue
+            assert self.report
 
             if self.report["ProblemType"] == "Hang":
                 self.finish_hang(f)
@@ -456,6 +434,7 @@ class UserInterface:
                 pass
             if not self.report and not self.load_report(report_file):
                 return
+            assert self.report
 
             if "Ignore" in self.report:
                 return
@@ -484,7 +463,7 @@ class UserInterface:
             allowed_to_report = apport.fileutils.allowed_to_report()
             response = self.ui_present_report_details(allowed_to_report)
             if response.report or response.examine:
-                if "Dependencies" not in self.report:
+                if "_MarkForUpload" not in self.report:
                     self.collect_info()
 
             if self.report is None:
@@ -604,7 +583,7 @@ class UserInterface:
         self.cur_package = apport.fileutils.find_file_package(path)
         self.report.add_os_info()
         allowed_to_report = apport.fileutils.allowed_to_report()
-        response = self.ui_present_report_details(allowed_to_report, modal_for=pid)
+        response = self.ui_present_report_details(allowed_to_report, modal_for=int(pid))
         if response.report:
             apport.fileutils.mark_hanging_process(self.report, pid)
             os.kill(int(pid), signal.SIGABRT)
@@ -635,7 +614,7 @@ class UserInterface:
         """Kill process with signal SIGSEGV."""
         os.kill(int(pid), signal.SIGSEGV)
 
-    def run_report_bug(self, symptom_script=None):
+    def run_report_bug(self, symptom_script: str | None = None) -> bool:
         # TODO: Split into smaller functions/methods
         # pylint: disable=too-many-branches,too-many-return-statements
         # pylint: disable=too-many-statements
@@ -762,7 +741,7 @@ class UserInterface:
 
         return True
 
-    def run_update_report(self):
+    def run_update_report(self) -> bool:
         """Update an existing bug with locally collected information."""
         # avoid irrelevant noise
         if not self.crashdb.can_update(self.args.update_report):
@@ -814,9 +793,7 @@ class UserInterface:
                 if not os.path.exists(
                     os.path.join(apport.report.PACKAGE_HOOK_DIR, f"source_{p}.py")
                 ):
-                    print(
-                        f"Package {p} not installed and no hook available," f" ignoring"
-                    )
+                    print(f"Package {p} not installed and no hook available, ignoring")
                     continue
             self.collect_info(ignore_uninstalled=True)
             info_collected = True
@@ -866,8 +843,8 @@ class UserInterface:
         """
         scripts = glob.glob(os.path.join(symptom_script_dir, "*.py"))
 
-        symptom_names = []
-        symptom_descriptions = []
+        # symptoms contains a list of (symptom_description, symptom_name)
+        symptoms = []
         for script in scripts:
             # scripts with an underscore can be used for private libraries
             if os.path.basename(script).startswith("_"):
@@ -886,28 +863,23 @@ class UserInterface:
                     "symptom script %s does not define run() function", script
                 )
                 continue
-            symptom_names.append(os.path.splitext(os.path.basename(script))[0])
-            symptom_descriptions.append(symb.get("description", symptom_names[-1]))
+            symptom_name = os.path.splitext(os.path.basename(script))[0]
+            symptoms.append((symb.get("description", symptom_name), symptom_name))
 
-        if not symptom_names:
+        if not symptoms:
             return False
 
-        symptom_descriptions, symptom_names = zip(
-            *sorted(zip(symptom_descriptions, symptom_names))
-        )
-        symptom_descriptions = list(symptom_descriptions)
-        symptom_names = list(symptom_names)
-        symptom_names.append(None)
-        symptom_descriptions.append("Other problem")
+        symptoms.sort()
+        symptoms.append(("Other problem", None))
 
         ch = self.ui_question_choice(
             _("What kind of problem do you want to report?"),
-            symptom_descriptions,
+            [description for description, name in symptoms],
             False,
         )
 
         if ch is not None:
-            symptom = symptom_names[ch[0]]
+            symptom = symptoms[ch[0]][1]
             if symptom:
                 self.run_report_bug(os.path.join(symptom_script_dir, symptom + ".py"))
             else:
@@ -1011,7 +983,7 @@ class UserInterface:
     #
 
     @staticmethod
-    def parse_argv_update(argv: typing.Sequence[str]) -> argparse.Namespace:
+    def parse_argv_update(argv: Sequence[str]) -> argparse.Namespace:
         """Parse command line options when being invoked in update mode."""
         parser = argparse.ArgumentParser(usage=_("%(prog)s <report number>"))
         parser.add_argument("-p", "--package", help=_("Specify package name."))
@@ -1259,28 +1231,26 @@ class UserInterface:
 
     def get_complete_size(self):
         """Return the size of the complete report."""
+        assert self.report
         # report wasn't loaded, so count manually
         size = 0
         for k in self.report:
             if self.report[k]:
                 try:
-                    # if we have a compressed value, take its size, but take
-                    # base64 overhead into account
-                    size += len(self.report[k].gzipvalue) * 8 / 6
+                    size += self.report[k].get_on_disk_size()
                 except AttributeError:
                     size += len(self.report[k])
         return size
 
     def get_reduced_size(self):
         """Return the size of the reduced report."""
+        assert self.report
         size = 0
         for k in self.report:
             if k != "CoreDump":
                 if self.report[k]:
                     try:
-                        # if we have a compressed value, take its size,
-                        # but take base64 overhead into account
-                        size += len(self.report[k].gzipvalue) * 8 / 6
+                        size += self.report[k].get_on_disk_size()
                     except AttributeError:
                         size += len(self.report[k])
 
@@ -1302,7 +1272,7 @@ class UserInterface:
 
     def restart(self):
         """Reopen the crashed application."""
-        assert "ProcCmdline" in self.report
+        assert self.report and "ProcCmdline" in self.report
 
         if os.fork() == 0:
             os.setsid()
@@ -1316,6 +1286,7 @@ class UserInterface:
 
     def examine(self):
         """Locally examine crash report."""
+        assert self.report_file
         response = self.ui_question_choice(
             _(
                 "This will launch apport-retrace in a terminal window"
@@ -1346,12 +1317,8 @@ class UserInterface:
 
         self.ui_run_terminal(cmds[response[0]])
 
-    def remember_send_report(self, send_report):
+    def remember_send_report(self, send_report: bool) -> None:
         """Put whoopsie in auto or never mode."""
-        if send_report:
-            send_report = "true"
-        else:
-            send_report = "false"
         try:
             subprocess.check_output(
                 [
@@ -1364,7 +1331,7 @@ class UserInterface:
                     "/com/ubuntu/WhoopsiePreferences",
                     "-m",
                     "com.ubuntu.WhoopsiePreferences.SetReportCrashes",
-                    send_report,
+                    str(send_report).lower(),
                 ]
             )
             subprocess.check_output(
@@ -1393,7 +1360,7 @@ class UserInterface:
 
     def check_report_crashdb(self):
         """Process reports' CrashDB field, if present."""
-        if "CrashDB" not in self.report:
+        if self.report is None or "CrashDB" not in self.report:
             return True
 
         # specification?
@@ -1447,6 +1414,7 @@ class UserInterface:
         If a symptom script is given, this will be run first (used by
         run_symptom()).
         """
+        assert self.report
         self.report["_MarkForUpload"] = "True"
 
         # skip if we already ran (we might load a processed report)
@@ -1480,9 +1448,9 @@ class UserInterface:
                         "This problem report applies to a program"
                         " which is not installed any more."
                     )
-                    self.report[
-                        "UnreportableReason"
-                    ] = f"{msg} ({self.report['InterpreterPath']})"
+                    self.report["UnreportableReason"] = (
+                        f"{msg} ({self.report['InterpreterPath']})"
+                    )
                     if on_finished:
                         on_finished()
                     return
@@ -1732,6 +1700,7 @@ class UserInterface:
             on_finished()
 
     def _is_snap(self):
+        assert self.report
         return "SnapSource" in self.report or "Snap" in self.report
 
     def open_url(self, url):
@@ -1739,41 +1708,26 @@ class UserInterface:
 
         Display an error dialog if everything fails.
         """
-        (r, w) = os.pipe()
-        if os.fork() > 0:
-            os.close(w)
-            status = os.wait()[1]
-            if status:
-                title = _("Unable to start web browser")
-                error = _("Unable to start web browser to open %s.") % url
-                message = os.fdopen(r).readline()
-                if message:
-                    error += "\n" + message
-                self.ui_error_message(title, error)
-            try:
-                os.close(r)
-            except OSError:
-                pass
-            return
-
-        os.setsid()
-        os.close(r)
-
         try:
             try:
                 run_as_real_user(["xdg-open", url], get_user_env=True)
+                return
             except OSError:
                 # fall back to webbrowser
-                webbrowser.open(url, new=True, autoraise=True)
-                sys.exit(0)
+                if webbrowser.open(url, new=1, autoraise=True):
+                    return
+                error_details = ""
         except Exception as error:  # pylint: disable=broad-except
-            os.write(w, str(error))
-            sys.exit(1)
-        os._exit(0)  # pylint: disable=protected-access
+            error_details = f"\n{error}"
+
+        title = _("Unable to start web browser")
+        message = _("Unable to start web browser to open %s.") % url
+        self.ui_error_message(title, message + error_details)
 
     def file_report(self):
         """Upload the current report and guide the user to the reporting
         web page."""
+        assert self.report
         # FIXME: This behaviour is not really correct, but necessary as
         # long as we only support a single crashdb and have whoopsie
         # hardcoded. Once we have multiple crash dbs, we need to check
@@ -1898,6 +1852,7 @@ class UserInterface:
 
         If so, display an info message and return True.
         """
+        assert self.report
         if not self.crashdb.accepts(self.report):
             return False
         if "UnreportableReason" in self.report:
@@ -1922,6 +1877,7 @@ class UserInterface:
 
         Return None if report cannot be associated to a .desktop file.
         """
+        assert self.report
         if "DesktopFile" in self.report and os.path.exists(self.report["DesktopFile"]):
             desktop_file = self.report["DesktopFile"]
         else:
@@ -1954,6 +1910,7 @@ class UserInterface:
         If so, tell the user about it, open the existing bug in a browser, and
         return True.
         """
+        assert self.report
         if not self.crashdb.accepts(self.report):
             return False
         if "_KnownReport" not in self.report:
@@ -1992,7 +1949,7 @@ class UserInterface:
     #
 
     def ui_present_report_details(
-        self, allowed_to_report: bool = True, modal_for: Optional[str] = None
+        self, allowed_to_report: bool = True, modal_for: int | None = None
     ) -> Action:
         """Show details of the bug report.
 
@@ -2038,7 +1995,7 @@ class UserInterface:
         """
         raise NotImplementedError("this function must be overridden by subclasses")
 
-    def ui_set_upload_progress(self, progress: Optional[float]) -> None:
+    def ui_set_upload_progress(self, progress: float | None) -> None:
         """Update data upload progress bar.
 
         Set the progress bar in the debug data upload progress window to the
