@@ -12,6 +12,7 @@ import sys
 import tempfile
 import unittest
 import unittest.mock
+from unittest.mock import MagicMock
 
 import apport.hookutils
 import apport.report
@@ -207,8 +208,7 @@ class T(unittest.TestCase):
             self.assertNotEqual(len(apport.hookutils.recent_syslog(re.compile("."))), 0)
 
     @unittest.mock.patch(
-        "apport.hookutils._root_command_prefix",
-        unittest.mock.MagicMock(return_value=[]),
+        "apport.hookutils._root_command_prefix", MagicMock(return_value=[])
     )
     def test_attach_mac_events(self):
         # TODO: Split into separate test cases
@@ -369,9 +369,7 @@ class T(unittest.TestCase):
         sys.stderr.write(f"[Δ {delta_kb} kB] ")
         self.assertLess(delta_kb, 5000)
 
-        self.assertTrue(
-            data.startswith("Apr 20 11:30:00 komputer kernel: bogus message\n")
-        )
+        self.assertRegex(data, "^Apr 20 11:30:00 komputer kernel: bogus message\n")
         self.assertGreater(len(data), 100000)
         self.assertLess(len(data), 1000000)
 
@@ -391,7 +389,7 @@ class T(unittest.TestCase):
         report = apport.report.Report()
         self.assertTrue(apport.hookutils.in_session_of_problem(report))
 
-        self.assertEqual(apport.hookutils.in_session_of_problem({}), None)
+        self.assertIsNone(apport.hookutils.in_session_of_problem({}))
 
         orig_ctime = locale.getlocale(locale.LC_TIME)
         try:
@@ -473,8 +471,7 @@ GdkPixbuf-CRITICAL **: gdk_pixbuf_scale_simple: another standard glib assertion
 
     @staticmethod
     @unittest.mock.patch(
-        "apport.hookutils._root_command_prefix",
-        unittest.mock.MagicMock(return_value=[]),
+        "apport.hookutils._root_command_prefix", MagicMock(return_value=[])
     )
     def test_no_crashes():
         """Functions do not crash (very shallow)."""
@@ -487,10 +484,6 @@ GdkPixbuf-CRITICAL **: gdk_pixbuf_scale_simple: another standard glib assertion
         apport.hookutils.attach_conffiles(report, "bash")
         apport.hookutils.attach_conffiles(report, "apport")
         apport.hookutils.attach_conffiles(report, "nonexisting")
-        apport.hookutils.attach_upstart_overrides(report, "apport")
-        apport.hookutils.attach_upstart_overrides(report, "nonexisting")
-        apport.hookutils.attach_upstart_logs(report, "apport")
-        apport.hookutils.attach_upstart_logs(report, "nonexisting")
         apport.hookutils.attach_default_grub(report)
 
     def test_command_output(self):
