@@ -7,6 +7,8 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
+"""Integration tests for the Java crash collection support."""
+
 import os
 import shutil
 import subprocess
@@ -20,7 +22,9 @@ from tests.paths import get_data_directory, local_test_environment
 
 
 @skip_if_command_is_missing("java")
-class T(unittest.TestCase):
+class TestJavaCrashes(unittest.TestCase):
+    """Integration tests for the Java crash collection support."""
+
     def setUp(self):
         self.env = os.environ | local_test_environment()
         datadir = get_data_directory()
@@ -56,18 +60,14 @@ class T(unittest.TestCase):
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
         )
-        self.assertNotEqual(
-            java.returncode, 0, "crash must exit with nonzero code"
-        )
+        self.assertNotEqual(java.returncode, 0, "crash must exit with nonzero code")
         self.assertIn("Can't catch this", java.stderr.decode())
 
         self._check_crash_report(str(crash_class))
 
     def test_crash_jar(self):
         """Crash in a .jar file."""
-        self.assertTrue(
-            self.crash_jar_path.exists(), f"{self.crash_jar_path} missing"
-        )
+        self.assertTrue(self.crash_jar_path.exists(), f"{self.crash_jar_path} missing")
         java = subprocess.run(
             [
                 "java",
@@ -80,9 +80,7 @@ class T(unittest.TestCase):
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
         )
-        self.assertNotEqual(
-            java.returncode, 0, "crash must exit with nonzero code"
-        )
+        self.assertNotEqual(java.returncode, 0, "crash must exit with nonzero code")
         self.assertIn("Can't catch this", java.stderr.decode())
 
         self._check_crash_report(f"{self.crash_jar_path}!/crash.class")
@@ -91,19 +89,19 @@ class T(unittest.TestCase):
         """Check that we have one crash report, and verify its contents."""
         reports = apport.fileutils.get_new_reports()
         self.assertEqual(len(reports), 1, "did not create a crash report")
-        r = apport.report.Report()
-        with open(reports[0], "rb") as f:
-            r.load(f)
-        self.assertEqual(r["ProblemType"], "Crash")
-        self.assertTrue(r["ProcCmdline"].startswith("java -classpath"), r)
+        report = apport.report.Report()
+        with open(reports[0], "rb") as report_file:
+            report.load(report_file)
+        self.assertEqual(report["ProblemType"], "Crash")
+        self.assertTrue(report["ProcCmdline"].startswith("java -classpath"), report)
         self.assertTrue(
-            r["StackTrace"].startswith(
+            report["StackTrace"].startswith(
                 "java.lang.RuntimeException: Can't catch this"
             )
         )
         if ".jar!" in main_file:
-            self.assertEqual(r["MainClassUrl"], "jar:file:" + main_file)
+            self.assertEqual(report["MainClassUrl"], f"jar:file:{main_file}")
         else:
-            self.assertEqual(r["MainClassUrl"], "file:" + main_file)
-        self.assertIn("DistroRelease", r)
-        self.assertIn("ProcCwd", r)
+            self.assertEqual(report["MainClassUrl"], f"file:{main_file}")
+        self.assertIn("DistroRelease", report)
+        self.assertIn("ProcCwd", report)
