@@ -6,20 +6,21 @@
 import gzip
 import os
 import pathlib
+import platform
 import shutil
 import subprocess
 import tempfile
 import time
 import unittest
 
-from apport.packaging_impl.apt_dpkg import impl
+from apport.packaging_impl.apt_dpkg import WITH_DEB822_SUPPORT, impl
 from tests.helper import skip_if_command_is_missing
 
 
 @skip_if_command_is_missing("dpkg")
 class T(unittest.TestCase):
     # pylint: disable=missing-class-docstring,missing-function-docstring
-    # pylint: disable=protected-access
+    # pylint: disable=protected-access,too-many-public-methods
 
     def setUp(self):
         # save and restore configuration file
@@ -87,9 +88,8 @@ class T(unittest.TestCase):
         self.assertTrue(impl.get_available_version("libc6").startswith("2"))
         self.assertRaises(ValueError, impl.get_available_version, "nonexisting")
 
-    def test_get_dependencies(self):
-        """get_dependencies()."""
-        # package with both Depends: and Pre-Depends:
+    def test_get_dependencies_depends_and_pre_depends(self):
+        """get_dependencies() on package with both Depends and Pre-Depends."""
         d = impl.get_dependencies("bash")
         self.assertGreater(len(d), 2)
         self.assertIn("libc6", d)
@@ -99,16 +99,18 @@ class T(unittest.TestCase):
                 continue
             self.assertTrue(impl.get_version(dep))
 
-        # Pre-Depends: only
+    def test_get_dependencies_pre_depends_only(self):
+        """get_dependencies() on package with Pre-Depends only."""
         d = impl.get_dependencies("coreutils")
         self.assertGreaterEqual(len(d), 1)
         self.assertIn("libc6", d)
         for dep in d:
             self.assertTrue(impl.get_version(dep))
 
-        # Depends: only
-        d = impl.get_dependencies("libc-bin")
-        self.assertGreaterEqual(len(d), 1)
+    def test_get_dependencies_depends_only(self):
+        """get_dependencies() on package with Depends only."""
+        d = impl.get_dependencies("sysvinit-utils")
+        self.assertIn("libc6", d)
         for dep in d:
             self.assertTrue(impl.get_version(dep))
 
@@ -151,14 +153,14 @@ class T(unittest.TestCase):
     def test_get_files(self):
         """get_files()."""
         self.assertRaises(ValueError, impl.get_files, "nonexisting")
-        self.assertIn("/bin/bash", impl.get_files("bash"))
+        self.assertIn("/usr/share/man/man1/bash.1.gz", impl.get_files("bash"))
 
     def test_get_file_package(self):
         """get_file_package() on installed files."""
-        self.assertEqual(impl.get_file_package("/bin/bash"), "bash")
-        self.assertEqual(impl.get_file_package("/bin/cat"), "coreutils")
+        self.assertEqual(impl.get_file_package("/usr/bin/bash"), "bash")
+        self.assertEqual(impl.get_file_package("/usr/bin/cat"), "coreutils")
         self.assertEqual(impl.get_file_package("/etc/pam.conf"), "libpam-runtime")
-        self.assertEqual(impl.get_file_package("/nonexisting"), None)
+        self.assertIsNone(impl.get_file_package("/nonexisting"))
 
     def test_get_file_package_uninstalled(self):
         """get_file_package() on uninstalled packages."""
@@ -198,12 +200,12 @@ bin/true                                                admin/superutils
             # use this as a mirror
             impl.set_mirror(f"file://{basedir}")
 
-            self.assertEqual(impl.get_file_package("usr/bin/frob", False), None)
+            self.assertIsNone(impl.get_file_package("usr/bin/frob", False))
             # must not match frob (same file name prefix)
             self.assertEqual(impl.get_file_package("usr/bin/frob", True), "frob-utils")
             self.assertEqual(impl.get_file_package("/usr/bin/frob", True), "frob-utils")
             # find files from -updates pocket
-            self.assertEqual(impl.get_file_package("/lib/libnew.so.5", False), None)
+            self.assertIsNone(impl.get_file_package("/lib/libnew.so.5", False))
             self.assertEqual(impl.get_file_package("/lib/libnew.so.5", True), "libnew5")
 
             # invalid mirror
@@ -219,14 +221,14 @@ bin/true                                                admin/superutils
             )
             cache_dir_files = sorted(os.listdir(cache_dir))
             self.assertEqual(len(cache_dir_files), 3)
-            self.assertEqual(impl.get_file_package("/bo/gu/s", True, cache_dir), None)
+            self.assertIsNone(impl.get_file_package("/bo/gu/s", True, cache_dir))
 
             # valid cache, should not need to access the mirror
             impl.set_mirror("file:///foo/nonexisting")
             self.assertEqual(
                 impl.get_file_package("/bin/true", True, cache_dir), "superutils"
             )
-            self.assertEqual(impl.get_file_package("/bo/gu/s", True, cache_dir), None)
+            self.assertIsNone(impl.get_file_package("/bo/gu/s", True, cache_dir))
             self.assertEqual(
                 impl.get_file_package("/lib/libnew.so.5", True, cache_dir), "libnew5"
             )
@@ -285,7 +287,7 @@ usr/bin/frob                                            foo/frob
             impl.set_mirror(f"file://{basedir}")
 
             # must not match system architecture
-            self.assertEqual(impl.get_file_package("usr/bin/frob", False), None)
+            self.assertIsNone(impl.get_file_package("usr/bin/frob", False))
             # must match correct architecture
             self.assertEqual(
                 impl.get_file_package("usr/bin/frob", True, arch="even"), "frob-utils"
@@ -426,13 +428,42 @@ usr/bin/frob                                            foo/frob
                 # pick first diversion we have
                 break
         else:  # pragma: no cover
-            self.fail(f"No non-local diversion found. dpkg-divert output: {output}")
+            self.skipTest(f"No non-local diversion found. dpkg-divert output: {output}")
 
         file = fields[2]
         pkg = fields[-1]
 
         self.assertEqual(impl.get_file_package(file), pkg)
 
+    @unittest.skipUnless(WITH_DEB822_SUPPORT, "deb822 not supported on this system")
+    def test_mirror_from_deb822_apt_sources(self) -> None:
+        sources_list_d = pathlib.Path(self.workdir) / "sources.list.d"
+        sources_list_d.mkdir()
+        info = platform.freedesktop_os_release()
+        primary_sources = sources_list_d / f"{info['ID']}.sources"
+        ppa_sources = sources_list_d / "0ppa.sources"
+        primary_sources.write_text(
+            """\
+Types: deb
+URIs: http://primary-mirror.example.com/distro/
+Suites: tuxy
+Components: main
+"""
+        )
+        ppa_sources.write_text(
+            """\
+Types: deb
+URIs: https://ppa.example.net/user/ppa-name/distro/
+Suites: lazy
+Components: main
+"""
+        )
+
+        self.assertEqual(
+            impl._get_primary_mirror_from_apt_sources(self.workdir),
+            "http://primary-mirror.example.com/distro/",
+        )
+
     def test_mirror_from_apt_sources(self):
         s = os.path.join(self.workdir, "sources.list")
 
