@@ -306,6 +306,7 @@ class T(unittest.TestCase):
         pr = problem_report.ProblemReport()
         pr.load(out, binary="compressed")
         self.assertEqual(pr["File"].get_value(), data)
+        self.assertEqual(pr["File"].name, "File")
 
     def test_size_limit(self):
         """Write and a big random file with a size limit key."""
@@ -454,7 +455,7 @@ class T(unittest.TestCase):
         self.assertTrue(not parts[1].is_multipart())
         self.assertEqual(parts[1].get_content_type(), "text/plain")
         self.assertEqual(parts[1].get_content_charset(), "utf-8")
-        self.assertEqual(parts[1].get_filename(), None)
+        self.assertIsNone(parts[1].get_filename())
         self.assertEqual(
             parts[1].get_payload(decode=True),
             b"ProblemType: Crash\nContext: Test suite\nDate: now!\n",
@@ -516,7 +517,7 @@ class T(unittest.TestCase):
         self.assertTrue(not parts[1].is_multipart())
         self.assertEqual(parts[1].get_content_type(), "text/plain")
         self.assertEqual(parts[1].get_content_charset(), "utf-8")
-        self.assertEqual(parts[1].get_filename(), None)
+        self.assertIsNone(parts[1].get_filename())
         self.assertEqual(
             parts[1].get_payload(decode=True),
             textwrap.dedent(
@@ -534,7 +535,7 @@ class T(unittest.TestCase):
 
     @staticmethod
     def decode_gzipped_message(message: email.message.Message) -> bytes:
-        with tempfile.TemporaryFile() as payload:
-            payload.write(message.get_payload(decode=True))
-            payload.seek(0)
-            return gzip.GzipFile(mode="rb", fileobj=payload).read()
+        payload = message.get_payload(decode=True)
+        assert isinstance(payload, bytes)
+        with gzip.GzipFile(mode="rb", fileobj=io.BytesIO(payload)) as gzip_file:
+            return gzip_file.read()
