@@ -8,6 +8,7 @@ import atexit
 import grp
 import io
 import os
+import re
 import shutil
 import subprocess
 import sys
@@ -16,6 +17,7 @@ import textwrap
 import time
 import unittest
 import unittest.mock
+from unittest.mock import MagicMock
 
 import apport.packaging
 import apport.report
@@ -49,7 +51,7 @@ class T(unittest.TestCase):
             elapsed_time += 0.1
 
         self.fail(
-            f"/proc/{pid}/cmdline not readable within" f" {int(elapsed_time)} seconds."
+            f"/proc/{pid}/cmdline not readable within {int(elapsed_time)} seconds."
         )
 
     @unittest.mock.patch("time.sleep")
@@ -121,13 +123,24 @@ class T(unittest.TestCase):
             self.assertLess(grp.getgrnam(g).gr_gid, 1000)
         self.assertNotIn(grp.getgrgid(os.getgid()).gr_name, pr["UserGroups"])
 
+    @unittest.mock.patch("os.geteuid")
+    def test_add_user_info_missing_user(self, geteuid_mock: MagicMock) -> None:
+        """Test add_user_info() with the effective user set to an dynamic user."""
+        geteuid_mock.return_value = 1000042
+        report = apport.report.Report()
+
+        report.add_user_info()
+
+        self.assertNotIn("UserGroups", report)
+        geteuid_mock.assert_called_once_with()
+
     def test_add_proc_info(self):
         # TODO: Split into separate test cases
         # pylint: disable=too-many-statements
         """add_proc_info()."""
         # check without additional safe environment variables
         pr = apport.report.Report()
-        self.assertEqual(pr.pid, None)
+        self.assertIsNone(pr.pid)
         pr.add_proc_info()
         self.assertEqual(pr.pid, os.getpid())
         self.assertTrue(
@@ -142,7 +155,7 @@ class T(unittest.TestCase):
             self.assertNotIn("LANG=", pr["ProcEnviron"])
         self.assertNotIn("USER", pr["ProcEnviron"])
         self.assertNotIn("PWD", pr["ProcEnviron"])
-        self.assertRegex(pr["ExecutablePath"], r".*\.py$")
+        self.assertRegex(pr["ExecutablePath"], rf"^({re.escape(sys.argv[0])}|.*\.py)$")
         self.assertEqual(
             int(pr["ExecutableTimestamp"]), int(os.stat(pr["ExecutablePath"]).st_mtime)
         )
@@ -252,7 +265,7 @@ class T(unittest.TestCase):
         # test process is gone, should complain about nonexisting PID
         self.assertRaises(ValueError, pr.add_proc_info, process.pid)
 
-    def test_add_proc_info_nonascii(self):
+    def test_add_proc_info_nonascii(self) -> None:
         """add_proc_info() for non-ASCII values"""
         lang = b"n\xc3\xb6_v\xc3\xb8lid"
 
@@ -297,7 +310,7 @@ class T(unittest.TestCase):
         self.assertEqual(r["ProcEnviron"], "LANG=xx_YY.UTF-8")
         self.assertEqual(r["CurrentDesktop"], "Pixel Pusher")
 
-    def test_add_path_classification(self):
+    def test_add_path_classification(self) -> None:
         """Classification of $PATH."""
         # system default
         with subprocess.Popen(
@@ -824,8 +837,7 @@ int main() { return f(42); }
         self.assertRaises(FileNotFoundError, pr.add_gdb_info)
 
     @unittest.mock.patch(
-        "apport.hookutils._root_command_prefix",
-        unittest.mock.MagicMock(return_value=[]),
+        "apport.hookutils._root_command_prefix", MagicMock(return_value=[])
     )
     def test_add_zz_parse_segv_details(self):
         """parse-segv produces sensible results"""
@@ -854,7 +866,7 @@ int main() { return f(42); }
 
         pr.add_proc_info()
         pr.add_hooks_info()
-        if pr["Architecture"] in ["amd64", "i386"]:
+        if pr["Architecture"] in {"amd64", "i386"}:
             # data/general-hooks/parse_segv.py only runs for x86 and x86_64
             self.assertIn("not located in a known VMA region", pr["SegvAnalysis"])
 
@@ -1161,7 +1173,7 @@ int main() { return f(42); }
 
             # negative match cases
             r_bash["Package"] = "bash-static 1-2"
-            self.assertEqual(r_bash.search_bug_patterns(pattern_url), None)
+            self.assertIsNone(r_bash.search_bug_patterns(pattern_url))
             r_bash["Package"] = "bash 1-21"
             self.assertEqual(
                 r_bash.search_bug_patterns(pattern_url),
@@ -1260,6 +1272,7 @@ int main() { return f(42); }
                         """\
                         import sys
                         def add_info(report):
+                            report['Order'] = f'{report.get("Order", "")} foo'
                             report['Field1'] = 'Field 1'
                             report['Field2'] = 'Field 2\\nBla'
                             if 'Spethial' in report:
@@ -1277,6 +1290,7 @@ int main() { return f(42); }
                     textwrap.dedent(
                         """\
                         def add_info(report):
+                            report['Order'] = f'{report.get("Order", "")} foo1'
                             report['CommonField1'] = 'CommonField 1'
                             if report['Package'] == 'commonspethial':
                                 raise StopIteration
@@ -1292,6 +1306,7 @@ int main() { return f(42); }
                     textwrap.dedent(
                         """\
                         def add_info(report):
+                            report['Order'] = f'{report.get("Order", "")} foo2'
                             report['CommonField2'] = 'CommonField 2'
                         """
                     )
@@ -1305,6 +1320,7 @@ int main() { return f(42); }
                     textwrap.dedent(
                         """\
                         def add_info(report, ui):
+                            report['Order'] = f'{report.get("Order", "")} foo3'
                             report['CommonField3'] = str(ui)
                         """
                     )
@@ -1320,6 +1336,7 @@ int main() { return f(42); }
                     textwrap.dedent(
                         """\
                         def add_info(report):
+                            report['Order'] = f'{report.get("Order", "")} notme'
                             report['BadField'] = 'XXX'
                         """
                     )
@@ -1338,6 +1355,7 @@ int main() { return f(42); }
                         "CommonField1",
                         "CommonField2",
                         "CommonField3",
+                        "Order",
                     ]
                 ),
                 "report has required fields",
@@ -1357,6 +1375,7 @@ int main() { return f(42); }
                         "CommonField1",
                         "CommonField2",
                         "CommonField3",
+                        "Order",
                     ]
                 ),
                 "report has required fields",
@@ -1377,6 +1396,7 @@ int main() { return f(42); }
                         "CommonField1",
                         "CommonField2",
                         "CommonField3",
+                        "Order",
                     ]
                 ),
                 "report has required fields",
@@ -1386,6 +1406,7 @@ int main() { return f(42); }
             self.assertEqual(r["CommonField1"], "CommonField 1")
             self.assertEqual(r["CommonField2"], "CommonField 2")
             self.assertEqual(r["CommonField3"], "NoninteractiveHookUI()")
+            self.assertEqual(r["Order"].strip(), "foo1 foo2 foo3 foo")
 
             r = apport.report.Report()
             r["Package"] = "foo 4.5-6"
@@ -1402,6 +1423,7 @@ int main() { return f(42); }
                         "CommonField1",
                         "CommonField2",
                         "CommonField3",
+                        "Order",
                     ]
                 ),
                 "report has required fields",
@@ -1410,6 +1432,7 @@ int main() { return f(42); }
             self.assertEqual(r["Field2"], "Field 2\nBla")
             self.assertEqual(r["CommonField1"], "CommonField 1")
             self.assertEqual(r["CommonField2"], "CommonField 2")
+            self.assertEqual(r["Order"].strip(), "foo1 foo2 foo3 foo")
 
             # test hook abort
             r["Spethial"] = "1"
@@ -1428,6 +1451,7 @@ int main() { return f(42); }
                     textwrap.dedent(
                         """\
                         def add_info(report, ui):
+                            report['Order'] = f'{report.get("Order", "")} source_foo'
                             report['Field1'] = 'Field 1'
                             report['Field2'] = 'Field 2\\nBla'
                             if report['Package'] == 'spethial':
@@ -1452,6 +1476,7 @@ int main() { return f(42); }
                         "CommonField1",
                         "CommonField2",
                         "CommonField3",
+                        "Order",
                     ]
                 ),
                 "report has required fields",
@@ -1461,6 +1486,7 @@ int main() { return f(42); }
             self.assertEqual(r["CommonField1"], "CommonField 1")
             self.assertEqual(r["CommonField2"], "CommonField 2")
             self.assertEqual(r["CommonField3"], "NoninteractiveHookUI()")
+            self.assertEqual(r["Order"].strip(), "foo1 foo2 foo3 source_foo")
 
             # test hook abort
             r["Package"] = "spethial"
@@ -1516,9 +1542,9 @@ int main() { return f(42); }
             r = apport.report.Report()
             r["Package"] = "foo-bin 0.2"
             r["SourcePackage"] = "foo"
-            r[
-                "ExecutablePath"
-            ] = f"{apport.report._opt_dir}/foolabs.example.com/foo/bin/frob"
+            r["ExecutablePath"] = (
+                f"{apport.report._opt_dir}/foolabs.example.com/foo/bin/frob"
+            )
 
             self.assertEqual(r.add_hooks_info(), False)
             self.assertEqual(r["SourceHook"], "1")
@@ -1803,55 +1829,11 @@ int main() { return f(42); }
         # real /proc/pid/maps and not just with our static test case above
         pr = apport.report.Report()
         pr.add_proc_info()
-        self.assertEqual(pr._address_to_offset(0), None)
+        self.assertIsNone(pr._address_to_offset(0))
         res = pr._address_to_offset(int(pr["ProcMaps"].split("-", 1)[0], 16) + 5)
         self.assertEqual(res.split("+", 1)[1], "5")
         self.assertIn("python", res.split("+", 1)[0])
 
-    def test_get_logind_session(self):
-        ret = apport.Report.get_logind_session(os.getpid())
-        if ret is None:
-            # ensure that we don't run under logind, and thus the None is
-            # justified
-            with open("/proc/self/cgroup", encoding="utf-8") as f:
-                contents = f.read()
-            sys.stdout.write("[not running under logind] ")
-            sys.stdout.flush()
-            self.assertNotIn("name=systemd:/user", contents)
-            return
-
-        (session, timestamp) = ret
-        self.assertNotEqual(session, "")
-        # session start must be >= 2014-01-01 and <= "now"
-        self.assertLess(timestamp, time.time())
-        self.assertGreater(
-            timestamp, time.mktime(time.strptime("2014-01-01", "%Y-%m-%d"))
-        )
-
-    def test_get_logind_session_fd(self):
-        proc_pid_fd = os.open(
-            f"/proc/{os.getpid()}", os.O_RDONLY | os.O_PATH | os.O_DIRECTORY
-        )
-        self.addCleanup(os.close, proc_pid_fd)
-        ret = apport.Report.get_logind_session(proc_pid_fd=proc_pid_fd)
-        if ret is None:
-            # ensure that we don't run under logind, and thus the None is
-            # justified
-            with open("/proc/self/cgroup", encoding="utf-8") as f:
-                contents = f.read()
-            sys.stdout.write("[not running under logind] ")
-            sys.stdout.flush()
-            self.assertNotIn("name=systemd:/user", contents)
-            return
-
-        (session, timestamp) = ret
-        self.assertNotEqual(session, "")
-        # session start must be >= 2014-01-01 and <= "now"
-        self.assertLess(timestamp, time.time())
-        self.assertGreater(
-            timestamp, time.mktime(time.strptime("2014-01-01", "%Y-%m-%d"))
-        )
-
     def test_command_output(self):
         out = apport.report._command_output(["echo", "hello"])
         self.assertEqual(out, "hello")
