@@ -15,8 +15,10 @@
 
 import argparse
 import collections
+import datetime
 import grp
 import os
+import pathlib
 import resource
 import shutil
 import signal
@@ -29,17 +31,27 @@ import textwrap
 import time
 import typing
 import unittest
-from typing import Optional
+from collections.abc import Callable
+from unittest.mock import MagicMock
 
 import psutil
 
 import apport.fileutils
-from tests.helper import import_module_from_file, read_shebang
+from tests.helper import import_module_from_file, read_shebang, run_test_executable
 from tests.paths import get_data_directory, local_test_environment
 
+try:
+    import systemd.journal
+
+    assert systemd.journal.Reader
+    SYSTEMD_IMPORT_ERROR = None
+except ImportError as import_error:
+    SYSTEMD_IMPORT_ERROR = import_error
+
 APPORT_PATH = get_data_directory() / "apport"
 apport_binary = import_module_from_file(APPORT_PATH)
 
+MAIL_UID = 8
 test_package = "coreutils"
 test_source = "coreutils"
 
@@ -64,6 +76,7 @@ class T(unittest.TestCase):
     # pylint: disable=protected-access,too-many-public-methods
     TEST_EXECUTABLE = os.path.realpath("/bin/sleep")
     TEST_ARGS = ["86400"]
+    maxDiff = None
 
     @classmethod
     def setUpClass(cls):
@@ -110,11 +123,11 @@ class T(unittest.TestCase):
         os.chdir("/tmp")
 
         # expected report name for test executable report
-        self.test_report = self._get_report_filename(self.TEST_EXECUTABLE)
+        self.test_report = None
 
     def tearDown(self):
         # permit tests to leave behind test_report, but nothing else
-        if os.path.exists(self.test_report):
+        if self.test_report and os.path.exists(self.test_report):
             apport.fileutils.delete_report(self.test_report)
         try:
             self.assertEqual(apport.fileutils.get_all_reports(), [])
@@ -670,23 +683,38 @@ class T(unittest.TestCase):
         # run test program in /run (which should only be writable to root)
         os.chdir("/run")
 
-        # run test program as user "mail"
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         # if a user can crash a suid root binary, it should not create
         # core files
-        self.do_crash(command=myexe, uid=8, suid_dumpable=2)
+        self.do_crash(command=myexe, uid=MAIL_UID, suid_dumpable=2)
+
+    @unittest.skipUnless(os.path.exists("/bin/ping"), "this test needs /bin/ping")
+    @unittest.skipIf(os.geteuid() != 0, "this test needs to be run as root")
+    def test_crash_suid_dumpable_debug(self):
+        """Report generation for setuid program with suid_dumpable set to 1.
+
+        ping has cap_net_raw=ep and therefore do_crash needs root.
+        """
+        resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
+
+        # if a user can crash a suid root binary, it should not create
+        # core files if /proc/sys/fs/suid_dumpable is set to 1 ("debug")
+        self.do_crash(
+            command="/bin/ping", args=["127.0.0.1"], uid=MAIL_UID, suid_dumpable=1
+        )
 
     @unittest.skipUnless(os.path.exists("/bin/ping"), "this test needs /bin/ping")
     @unittest.skipIf(os.geteuid() != 0, "this test needs to be run as root")
     def test_crash_setuid_drop(self):
         """Report generation for setuid program which drops root."""
-        # run ping as user "mail"
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         # if a user can crash a suid root binary, it should not create
         # core files
-        self.do_crash(command="/bin/ping", args=["127.0.0.1"], uid=8, suid_dumpable=2)
+        self.do_crash(
+            command="/bin/ping", args=["127.0.0.1"], uid=MAIL_UID, suid_dumpable=2
+        )
 
     @unittest.skipIf(os.geteuid() != 0, "this test needs to be run as root")
     def test_crash_setuid_unpackaged(self):
@@ -700,7 +728,6 @@ class T(unittest.TestCase):
         os.close(fd)
         os.chmod(myexe, 0o4755)
 
-        # run test program as user "mail"
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
 
         # if a user can crash a suid root binary, it should not create
@@ -709,7 +736,7 @@ class T(unittest.TestCase):
             command=myexe,
             expect_corefile=False,
             expect_report=False,
-            uid=8,
+            uid=MAIL_UID,
             suid_dumpable=2,
         )
 
@@ -742,11 +769,10 @@ class T(unittest.TestCase):
     def test_crash_setuid_drop_via_socket(self):
         """Report generation via socket for setuid program which drops root."""
         resource.setrlimit(resource.RLIMIT_CORE, (-1, -1))
-        # run ping as user "mail"
         self.do_crash(
             command="/bin/ping",
             args=["127.0.0.1"],
-            uid=8,
+            uid=MAIL_UID,
             suid_dumpable=2,
             via_socket=True,
         )
@@ -759,12 +785,13 @@ class T(unittest.TestCase):
         self.assertEqual(report["ExecutablePath"], os.path.realpath("/bin/ping"))
 
     @unittest.mock.patch("os.readlink")
-    def test_is_not_same_ns(self, readlink_mock: unittest.mock.MagicMock) -> None:
+    def test_is_not_same_ns(self, readlink_mock: MagicMock) -> None:
         readlink_mock.side_effect = ["mnt:[1]", "mnt:[2]"]
         open_mock = unittest.mock.mock_open(read_data="0::/user.slice\n")
         command = [self.TEST_EXECUTABLE] + self.TEST_ARGS
-        with subprocess.Popen(command) as test_process, unittest.mock.patch(
-            "builtins.open", open_mock
+        with (
+            subprocess.Popen(command) as test_process,
+            unittest.mock.patch("builtins.open", open_mock),
         ):
             try:
                 same_ns = apport_binary.is_same_ns(test_process.pid, "mnt")
@@ -781,6 +808,79 @@ class T(unittest.TestCase):
             f"/proc/{test_process.pid}/cgroup", encoding="utf-8"
         )
 
+    @unittest.skipIf(
+        SYSTEMD_IMPORT_ERROR,
+        f"systemd Python module not available: {SYSTEMD_IMPORT_ERROR}",
+    )
+    @unittest.mock.patch("systemd.journal.Reader.__iter__")
+    def test_crash_apport_from_systemd_coredump(self, reader_mock: MagicMock) -> None:
+        """Report generation with apport from systemd-coredump."""
+        with tempfile.TemporaryDirectory() as tmpdir, run_test_executable() as pid:
+            coredump_file = pathlib.Path(tmpdir) / "core.zst"
+            coredump_file.write_text("mocked core")
+
+            now = datetime.datetime.now(tz=datetime.timezone.utc)
+            expected_report = apport.report.Report(
+                date=now.strftime("%a %b %e %H:%M:%S %Y")
+            )
+            expected_report.pid = pid
+            expected_report.add_proc_info()
+            expected_report.add_user_info()
+            expected_report.add_os_info()
+            expected_report.add_package_info()
+            expected_report["Signal"] = "11"
+            expected_report["SignalName"] = "SIGSEGV"
+            expected_report["_HooksRun"] = "no"
+            # systemd-coredump does not collect /proc/<pid>/attr/current
+            expected_report.pop("ProcAttrCurrent", None)
+
+            systemd_coredump = {
+                "COREDUMP_CGROUP": "mocked cgroup",
+                "COREDUMP_CMDLINE": expected_report["ProcCmdline"],
+                "COREDUMP_COMM": self.TEST_EXECUTABLE,
+                "COREDUMP_CWD": expected_report["ProcCwd"],
+                "COREDUMP_ENVIRON": "".join(
+                    f"{k}={v}\n" for k, v in os.environ.items()
+                ),
+                "COREDUMP_EXE": self.TEST_EXECUTABLE,
+                "COREDUMP_FILENAME": str(coredump_file),
+                "COREDUMP_GID": os.getgid(),
+                "COREDUMP_HOSTNAME": "mocked hostname",
+                "COREDUMP_OPEN_FDS": "mocked open file descriptors",
+                "COREDUMP_OWNER_UID": os.getuid(),
+                "COREDUMP_PACKAGE_JSON": '{"elfType":"coredump"}',
+                "COREDUMP_PID": pid,
+                "COREDUMP_PROC_AUXV": b"mocked /proc/<pid>/auxv",
+                "COREDUMP_PROC_CGROUP": "mocked /proc/<pid>/cgroup",
+                "COREDUMP_PROC_LIMITS": "mocked /proc/<pid>/limits",
+                "COREDUMP_PROC_MAPS": expected_report["ProcMaps"],
+                "COREDUMP_PROC_MOUNTINFO": "mocked /proc/<pid>/mountinfo",
+                "COREDUMP_PROC_STATUS": expected_report["ProcStatus"],
+                "COREDUMP_RLIMIT": "9223372036854775808",
+                "COREDUMP_ROOT": "/",
+                "COREDUMP_SIGNAL": 11,
+                "COREDUMP_SIGNAL_NAME": "SIGSEGV",
+                "COREDUMP_SLICE": "mocked slice",
+                "COREDUMP_TIMESTAMP": now,
+                "COREDUMP_UID": os.getuid(),
+                "COREDUMP_UNIT": "mocked unit",
+                "COREDUMP_USER_UNIT": "mocked user unit",
+            }
+            reader_mock.return_value = iter([systemd_coredump])
+            self.test_report = self._get_report_filename(self.TEST_EXECUTABLE)
+
+            exit_code = apport_binary.main(["--from-systemd-coredump", "37-23489-42"])
+
+        self.assertEqual(exit_code, 0)
+        reader_mock.assert_called_once_with()
+        self._check_report()
+        report = apport.Report()
+        with open(self.test_report, "rb") as report_file:
+            report.load(report_file, binary="compressed")
+        self.assertEqual(report["CoreDump"].compressed_value, b"mocked core")
+        del report["CoreDump"]
+        self.assertEqual(dict(report), dict(expected_report))
+
     #
     # Helper methods
     #
@@ -812,9 +912,7 @@ class T(unittest.TestCase):
         orig_os_open = os.open
 
         def _mocked_os_open(
-            path: typing.Union[str, os.PathLike[str]],
-            flags: int,
-            dir_fd: Optional[int] = None,
+            path: str | os.PathLike[str], flags: int, dir_fd: int | None = None
         ) -> int:
             if path == "root/run/apport.socket":
                 return orig_os_open(socket_path, flags)
@@ -884,7 +982,7 @@ class T(unittest.TestCase):
         self.assertNotEqual(gdb.stdout.strip(), "")
 
     def _check_report(
-        self, expect_report: bool = True, expected_owner: Optional[int] = None
+        self, expect_report: bool = True, expected_owner: int | None = None
     ) -> None:
         if not expect_report:
             self.assertEqual(apport.fileutils.get_all_reports(), [])
@@ -915,6 +1013,7 @@ class T(unittest.TestCase):
             args = self.TEST_ARGS
 
         assert os.access(command, os.X_OK), f"{command} is not executable"
+        self.test_report = self._get_report_filename(command, uid)
 
         env = os.environ.copy()
         # set UTF-8 environment variable, to check proper parsing in apport
@@ -942,13 +1041,13 @@ class T(unittest.TestCase):
         self,
         expect_corefile: bool = False,
         sig: int = signal.SIGSEGV,
-        command: Optional[str] = None,
-        expected_command: Optional[str] = None,
-        uid: Optional[int] = None,
-        expect_corefile_owner: Optional[str] = None,
-        args: Optional[list[str]] = None,
+        command: str | None = None,
+        expected_command: str | None = None,
+        uid: int | None = None,
+        expect_corefile_owner: str | None = None,
+        args: list[str] | None = None,
         suid_dumpable: int = 1,
-        hook_before_apport: Optional[typing.Callable] = None,
+        hook_before_apport: Callable | None = None,
         expect_report: bool = True,
         via_socket: bool = False,
     ) -> None:
@@ -964,6 +1063,10 @@ class T(unittest.TestCase):
         The socket is being used in a container via systemd activation,
         where the core dump gets read from /run/apport.socket.
 
+        If the specified program has file capability or is a
+        set‐user‐ID/set-group-ID program, this method needs to be called
+        as root to allow GDB to call the program.
+
         Note: The arguments for the test program must not contain spaces.
         Since there was no need for it, the support was not implemented.
         """
@@ -981,7 +1084,7 @@ class T(unittest.TestCase):
             args.insert(0, command)
             command = shebang
 
-        self.test_report = self._get_report_filename(command)
+        self.test_report = self._get_report_filename(command, uid)
 
         gdb_core_file = os.path.join(self.workdir, "core")
         self.assertFalse(
@@ -1047,17 +1150,16 @@ class T(unittest.TestCase):
                 self._check_core_file_is_valid(core_path, command)
             finally:
                 os.unlink(core_path)
-        else:
-            if os.path.exists(core_path):
-                try:
-                    os.unlink(core_path)
-                except OSError as error:
-                    sys.stderr.write(
-                        f"WARNING: cannot clean up core file {core_path}:"
-                        f" {str(error)}\n"
-                    )
+        elif os.path.exists(core_path):
+            try:
+                os.unlink(core_path)
+            except OSError as error:
+                sys.stderr.write(
+                    f"WARNING: cannot clean up core file {core_path}:"
+                    f" {str(error)}\n"
+                )
 
-                self.fail("leaves unexpected core file behind")
+            self.fail("leaves unexpected core file behind")
 
         self._check_report(
             expect_report=expect_report,
@@ -1081,9 +1183,7 @@ class T(unittest.TestCase):
 
         args = " ".join(f" {a}" for a in args)
         if uid is not None:
-            args = (
-                f" --reuid={uid} --clear-groups " f"/bin/sh -c 'exec {command}{args}'"
-            )
+            args = f" --reuid={uid} --clear-groups /bin/sh -c 'exec {command}{args}'"
             command = "/usr/bin/setpriv"
             gdb_args += ["--ex", "set follow-fork-mode child"]
 
@@ -1097,10 +1197,12 @@ class T(unittest.TestCase):
         return gdb_args
 
     @staticmethod
-    def _get_report_filename(command: str) -> str:
+    def _get_report_filename(command: str, uid: int | None = None) -> str:
+        if uid is None:
+            uid = os.getuid()
         return os.path.join(
             apport.fileutils.report_dir,
-            f"{os.path.realpath(command).replace('/', '_')}" f".{os.getuid()}.crash",
+            f"{os.path.realpath(command).replace('/', '_')}.{uid}.crash",
         )
 
     def check_report_coredump(self, report_path):
@@ -1128,7 +1230,7 @@ class T(unittest.TestCase):
             timeout += 0.1
         else:
             self.fail(
-                f"Core file {core_file} not created " f"within {int(timeout)} seconds."
+                f"Core file {core_file} not created within {int(timeout)} seconds."
             )
 
         gdb_process = psutil.Process(gdb_pid)
@@ -1172,6 +1274,9 @@ class T(unittest.TestCase):
         sleep_mock.assert_called_with(0.1)
         self.assertEqual(sleep_mock.call_count, 600)
 
+    # False positive return statement for unittest.TestCase.fail
+    # See https://github.com/pylint-dev/pylint/issues/4167
+    # pylint: disable-next=inconsistent-return-statements
     def wait_for_gdb_child_process(self, gdb_pid: int, command: str) -> psutil.Process:
         """Wait until GDB execv()ed the child process."""
         gdb_process = psutil.Process(gdb_pid)
@@ -1199,18 +1304,26 @@ class T(unittest.TestCase):
     @unittest.mock.patch("time.sleep")
     def test_wait_for_gdb_child_process(self, sleep_mock):
         """Test wait_for_gdb_child_process() helper method."""
-        child = unittest.mock.MagicMock(spec=psutil.Process)
-        child.status.side_effect = ["tracing-stop", "sleeping"]
-        child.cmdline.return_value = [self.TEST_EXECUTABLE] + self.TEST_ARGS
+        child = MagicMock(spec=psutil.Process)
+        child.status.side_effect = ["tracing-stop", "running", "sleeping"]
+        child.cmdline.side_effect = [
+            ["/bin/sh", "-c", f"exec {self.TEST_EXECUTABLE}"],
+            [self.TEST_EXECUTABLE] + self.TEST_ARGS,
+        ]
         with unittest.mock.patch("psutil.Process", spec=psutil.Process) as process_mock:
             process_mock.return_value.children.side_effect = [
                 [],
                 [child],  # child not started (tracing-stop)
+                [child],  # shell wrapper running
                 [child],  # child ready
             ]
+
             self.wait_for_gdb_child_process(123456789, self.TEST_EXECUTABLE)
+
         sleep_mock.assert_called_with(0.1)
-        self.assertEqual(sleep_mock.call_count, 2)
+        self.assertEqual(sleep_mock.call_count, 3)
+        self.assertEqual(child.status.call_count, 3)
+        self.assertEqual(child.cmdline.call_count, 2)
 
     @unittest.mock.patch("psutil.Process", spec=psutil.Process)
     @unittest.mock.patch("time.sleep")
