@@ -272,7 +272,7 @@ def test_install_packages_unversioned(configdir, cachedir, rootdir, apt_style):
     assert "coreutils 8.32-4.1ubuntu1" in pkglist
     assert "coreutils-dbgsym 8.32-4.1ubuntu1" in pkglist
     assert "tzdata 2022a-0ubuntu1" in pkglist
-    assert len(pkglist), 3 == str(pkglist)
+    assert len(pkglist), str(pkglist) == 3
 
 
 @pytest.mark.skipif(not has_internet(), reason="online test")
@@ -507,7 +507,7 @@ def test_install_packages_permanent_sandbox_repack(
         permanent_rootdir=True,
     )
     curl_library = _get_library_path("libcurl.so", rootdir)
-    assert "libcurl-gnutls.so" == os.readlink(curl_library)
+    assert os.readlink(curl_library) == "libcurl-gnutls.so"
 
     impl.install_packages(
         rootdir,
@@ -518,7 +518,7 @@ def test_install_packages_permanent_sandbox_repack(
         cachedir,
         permanent_rootdir=True,
     )
-    assert "libcurl-nss.so" == os.readlink(curl_library)
+    assert os.readlink(curl_library) == "libcurl-nss.so"
 
     impl.install_packages(
         rootdir,
@@ -529,7 +529,7 @@ def test_install_packages_permanent_sandbox_repack(
         cachedir,
         permanent_rootdir=True,
     )
-    assert "libcurl-gnutls.so" == os.readlink(curl_library)
+    assert os.readlink(curl_library) == "libcurl-gnutls.so"
 
 
 @pytest.mark.skipif(not has_internet(), reason="online test")
@@ -850,7 +850,7 @@ def test_create_sources_for_an_unnamed_ppa(configdir, rootdir, apt_style):
         stdout=subprocess.PIPE,
     )
     apt_keys = gpg.stdout.decode()
-    assert "" == apt_keys
+    assert apt_keys == ""
 
 
 def test_use_sources_for_a_ppa(configdir, rootdir, apt_style):
@@ -933,7 +933,7 @@ def _ubuntu_archive_uri(arch=None):
     """Return archive URI for the given architecture."""
     if arch is None:
         arch = impl.get_system_architecture()
-    if arch in ("amd64", "i386"):
+    if arch in {"amd64", "i386"}:
         return "http://archive.ubuntu.com/ubuntu"
     return "http://ports.ubuntu.com/ubuntu-ports"
 
@@ -1078,7 +1078,7 @@ def _copy_ubuntu_keyrings(keyring_dir: str) -> None:
     dbgsym_keyring = "/usr/share/keyrings/ubuntu-dbgsym-keyring.gpg"
     if not os.path.isfile(dbgsym_keyring):
         pytest.skip(  # pragma: no cover
-            f"{dbgsym_keyring} missing. " f"Please install ubuntu-dbgsym-keyring!"
+            f"{dbgsym_keyring} missing. Please install ubuntu-dbgsym-keyring!"
         )
     # Convert from GPG keybox database format to OpenPGP Public Key format
     output_dbgsym_keyring = os.path.join(keyring_dir, os.path.basename(dbgsym_keyring))
