@@ -20,11 +20,11 @@ import textwrap
 import unittest
 import unittest.mock
 from gettext import gettext as _
+from unittest.mock import MagicMock
 
 try:
     from PyQt5.QtCore import QCoreApplication, QTimer
-    from PyQt5.QtGui import QIcon
-    from PyQt5.QtWidgets import QApplication, QProgressBar, QTreeWidget
+    from PyQt5.QtWidgets import QFileDialog, QProgressBar, QTreeWidget
 
     PYQT5_IMPORT_ERROR = None
 except ImportError as error:
@@ -46,6 +46,7 @@ else:
 @unittest.skipIf(PYQT5_IMPORT_ERROR, f"PyQt/PyKDE not available: {PYQT5_IMPORT_ERROR}")
 class T(unittest.TestCase):
     # pylint: disable=missing-class-docstring,missing-function-docstring
+    # pylint: disable=too-many-public-methods
     COLLECTING_DIALOG = unittest.mock.call(
         str(apport_kde_path.parent),
         "Collecting Problem Information",
@@ -68,11 +69,6 @@ class T(unittest.TestCase):
         os.environ["LANGUAGE"] = "C"
 
         cls.argv = [str(apport_kde_path)]
-        cls.app = QApplication(cls.argv)
-        cls.app.applicationName = "apport-kde"
-        cls.app.applicationDisplayName = _("Apport")
-        cls.app.windowIcon = QIcon.fromTheme("apport")
-
         r = apport.report.Report()
         r.add_os_info()
         cls.distro = r["DistroRelease"]
@@ -90,9 +86,9 @@ class T(unittest.TestCase):
         os.environ["APPORT_IGNORE_OBSOLETE_PACKAGES"] = "1"
         os.environ["APPORT_DISABLE_DISTRO_CHECK"] = "1"
 
-        self.app = MainUserInterface(self.argv)
+        self.ui = MainUserInterface(self.argv)
         # use in-memory crashdb
-        self.app.crashdb = apport.crashdb_impl.memory.CrashDatabase(None, {})
+        self.ui.crashdb = apport.crashdb_impl.memory.CrashDatabase(None, {})
 
         # disable package hooks, as they might ask for sudo password and other
         # interactive bits; allow tests to install their own hooks
@@ -101,20 +97,22 @@ class T(unittest.TestCase):
         apport.report.PACKAGE_HOOK_DIR = self.hook_dir
 
         # test report
-        self.app.report_file = os.path.join(self.report_dir, "bash.crash")
+        self.ui.report_file = os.path.join(self.report_dir, "bash.crash")
 
-        self.app.report = apport.report.Report()
-        self.app.report["ExecutablePath"] = "/bin/bash"
-        self.app.report["Signal"] = "11"
-        self.app.report["CoreDump"] = b"\x01\x02"
-        with open(self.app.report_file, "wb") as f:
-            self.app.report.write(f)
+        self.ui.report = apport.report.Report()
+        self.ui.report["ExecutablePath"] = "/bin/bash"
+        self.ui.report["Signal"] = "11"
+        self.ui.report["CoreDump"] = b"\x01\x02"
+        with open(self.ui.report_file, "wb") as f:
+            self.ui.report.write(f)
 
     def tearDown(self):
-        if self.app.dialog:
+        if self.ui.dialog:
             QCoreApplication.processEvents()
-            self.app.dialog.done(0)
+            self.ui.dialog.done(0)
             QCoreApplication.processEvents()
+        self.ui.app.exit()
+        del self.ui.app
 
         shutil.rmtree(self.report_dir)
         shutil.rmtree(self.hook_dir)
@@ -124,10 +122,10 @@ class T(unittest.TestCase):
         crash."""
 
         def c():
-            self.app.dialog.reject()
+            self.ui.dialog.reject()
 
         QTimer.singleShot(0, c)
-        result = self.app.ui_present_report_details(True)
+        result = self.ui.ui_present_report_details(True)
         self.assertFalse(result.report)
 
     def test_kernel_crash_layout(self):
@@ -141,21 +139,21 @@ class T(unittest.TestCase):
         | [ Show Details ]                                   [ Continue ] |
         +-----------------------------------------------------------------+
         """
-        self.app.report["ProblemType"] = "KernelCrash"
+        self.ui.report["ProblemType"] = "KernelCrash"
         QTimer.singleShot(0, QCoreApplication.quit)
-        self.app.ui_present_report_details(True)
-        self.assertEqual(self.app.dialog.windowTitle(), self.distro.split()[0])
+        self.ui.ui_present_report_details(True)
+        self.assertEqual(self.ui.dialog.windowTitle(), self.distro.split()[0])
         self.assertEqual(
-            self.app.dialog.heading.text(),
+            self.ui.dialog.heading.text(),
             _("Sorry, %s has experienced an internal error.") % self.distro,
         )
-        self.assertTrue(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertTrue(self.app.dialog.details.isVisible())
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Continue"))
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
-        self.assertFalse(self.app.dialog.text.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertTrue(self.ui.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Continue"))
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
+        self.assertFalse(self.ui.dialog.text.isVisible())
 
     def test_package_crash_layout(self):
         """Display crash dialog for a failed package installation.
@@ -169,33 +167,33 @@ class T(unittest.TestCase):
         | [ Show Details ]                                   [ Continue ] |
         +-----------------------------------------------------------------+
         """
-        self.app.report["ProblemType"] = "Package"
-        self.app.report["Package"] = "apport 1.2.3~0ubuntu1"
+        self.ui.report["ProblemType"] = "Package"
+        self.ui.report["Package"] = "apport 1.2.3~0ubuntu1"
         QTimer.singleShot(0, QCoreApplication.quit)
-        self.app.ui_present_report_details(True)
-        self.assertEqual(self.app.dialog.windowTitle(), self.distro.split()[0])
+        self.ui.ui_present_report_details(True)
+        self.assertEqual(self.ui.dialog.windowTitle(), self.distro.split()[0])
         self.assertEqual(
-            self.app.dialog.heading.text(),
+            self.ui.dialog.heading.text(),
             _("Sorry, a problem occurred while installing software."),
         )
-        self.assertTrue(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertTrue(self.app.dialog.details.isVisible())
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Continue"))
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertTrue(self.ui.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Continue"))
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
         self.assertEqual(
-            self.app.dialog.text.text(), _("Package: apport 1.2.3~0ubuntu1")
+            self.ui.dialog.text.text(), _("Package: apport 1.2.3~0ubuntu1")
         )
 
     def test_regular_crash_thread_layout(self):
         """A thread of execution has failed, but the application persists."""
-        self.app.report["ProblemType"] = "Crash"
-        self.app.report["ProcStatus"] = "Name:\tupstart\nPid:\t1"
+        self.ui.report["ProblemType"] = "Crash"
+        self.ui.report["ProcStatus"] = "Name:\tsystemd\nPid:\t1"
         QTimer.singleShot(0, QCoreApplication.quit)
-        self.app.ui_present_report_details(True)
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Continue"))
+        self.ui.ui_present_report_details(True)
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Continue"))
 
     def test_regular_crash_layout(self):
         """Display crash dialog for an application crash.
@@ -209,9 +207,9 @@ class T(unittest.TestCase):
         | [ Show Details ]                                   [ Continue ] |
         +-----------------------------------------------------------------+
         """
-        self.app.report["ProblemType"] = "Crash"
-        self.app.report["CrashCounter"] = "1"
-        self.app.report["Package"] = "apport 1.2.3~0ubuntu1"
+        self.ui.report["ProblemType"] = "Crash"
+        self.ui.report["CrashCounter"] = "1"
+        self.ui.report["Package"] = "apport 1.2.3~0ubuntu1"
         with tempfile.NamedTemporaryFile(mode="w+") as fp:
             fp.write(
                 textwrap.dedent(
@@ -224,26 +222,26 @@ class T(unittest.TestCase):
                 )
             )
             fp.flush()
-            self.app.report["DesktopFile"] = fp.name
+            self.ui.report["DesktopFile"] = fp.name
             QTimer.singleShot(0, QCoreApplication.quit)
-            self.app.ui_present_report_details(True)
-        self.assertEqual(self.app.dialog.windowTitle(), self.distro.split()[0])
+            self.ui.ui_present_report_details(True)
+        self.assertEqual(self.ui.dialog.windowTitle(), self.distro.split()[0])
         self.assertEqual(
-            self.app.dialog.heading.text(),
+            self.ui.dialog.heading.text(),
             _("The application Apport has closed unexpectedly."),
         )
-        self.assertTrue(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertTrue(self.app.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertTrue(self.ui.dialog.details.isVisible())
         # no ProcCmdline, cannot restart
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Continue"))
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
-        self.assertFalse(self.app.dialog.text.isVisible())
-        self.assertFalse(self.app.dialog.text.isVisible())
-        self.assertTrue(self.app.dialog.ignore_future_problems.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Continue"))
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
+        self.assertFalse(self.ui.dialog.text.isVisible())
+        self.assertFalse(self.ui.dialog.text.isVisible())
+        self.assertTrue(self.ui.dialog.ignore_future_problems.isVisible())
         self.assertTrue(
-            str(self.app.dialog.ignore_future_problems.text()).endswith(
+            str(self.ui.dialog.ignore_future_problems.text()).endswith(
                 "of this program version"
             )
         )
@@ -261,11 +259,11 @@ class T(unittest.TestCase):
         +-----------------------------------------------------------------+
         """
         # pretend we got called through run_crashes() which sets offer_restart
-        self.app.offer_restart = True
-        self.app.report["ProblemType"] = "Crash"
-        self.app.report["CrashCounter"] = "1"
-        self.app.report["ProcCmdline"] = "apport-bug apport"
-        self.app.report["Package"] = "apport 1.2.3~0ubuntu1"
+        self.ui.offer_restart = True
+        self.ui.report["ProblemType"] = "Crash"
+        self.ui.report["CrashCounter"] = "1"
+        self.ui.report["ProcCmdline"] = "apport-bug apport"
+        self.ui.report["Package"] = "apport 1.2.3~0ubuntu1"
         with tempfile.NamedTemporaryFile(mode="w+") as fp:
             fp.write(
                 textwrap.dedent(
@@ -278,25 +276,25 @@ class T(unittest.TestCase):
                 )
             )
             fp.flush()
-            self.app.report["DesktopFile"] = fp.name
+            self.ui.report["DesktopFile"] = fp.name
             QTimer.singleShot(0, QCoreApplication.quit)
-            self.app.ui_present_report_details(True)
-        self.assertEqual(self.app.dialog.windowTitle(), self.distro.split()[0])
+            self.ui.ui_present_report_details(True)
+        self.assertEqual(self.ui.dialog.windowTitle(), self.distro.split()[0])
         self.assertEqual(
-            self.app.dialog.heading.text(),
+            self.ui.dialog.heading.text(),
             _("The application Apport has closed unexpectedly."),
         )
-        self.assertTrue(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertTrue(self.app.dialog.details.isVisible())
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Relaunch"))
-        self.assertTrue(self.app.dialog.closed_button.isVisible())
-        self.assertFalse(self.app.dialog.text.isVisible())
-        self.assertFalse(self.app.dialog.text.isVisible())
-        self.assertTrue(self.app.dialog.ignore_future_problems.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertTrue(self.ui.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Relaunch"))
+        self.assertTrue(self.ui.dialog.closed_button.isVisible())
+        self.assertFalse(self.ui.dialog.text.isVisible())
+        self.assertFalse(self.ui.dialog.text.isVisible())
+        self.assertTrue(self.ui.dialog.ignore_future_problems.isVisible())
         self.assertTrue(
-            str(self.app.dialog.ignore_future_problems.text()).endswith(
+            str(self.ui.dialog.ignore_future_problems.text()).endswith(
                 "of this program version"
             )
         )
@@ -315,10 +313,10 @@ class T(unittest.TestCase):
         """
         # pretend we did not get called through run_crashes(),
         # thus no offer_restart
-        self.app.report["ProblemType"] = "Crash"
-        self.app.report["CrashCounter"] = "1"
-        self.app.report["ProcCmdline"] = "apport-bug apport"
-        self.app.report["Package"] = "apport 1.2.3~0ubuntu1"
+        self.ui.report["ProblemType"] = "Crash"
+        self.ui.report["CrashCounter"] = "1"
+        self.ui.report["ProcCmdline"] = "apport-bug apport"
+        self.ui.report["Package"] = "apport 1.2.3~0ubuntu1"
         with tempfile.NamedTemporaryFile(mode="w+") as fp:
             fp.write(
                 textwrap.dedent(
@@ -331,19 +329,19 @@ class T(unittest.TestCase):
                 )
             )
             fp.flush()
-            self.app.report["DesktopFile"] = fp.name
+            self.ui.report["DesktopFile"] = fp.name
             QTimer.singleShot(0, QCoreApplication.quit)
-            self.app.ui_present_report_details(True)
+            self.ui.ui_present_report_details(True)
         self.assertEqual(
-            self.app.dialog.heading.text(),
+            self.ui.dialog.heading.text(),
             _("The application Apport has closed unexpectedly."),
         )
-        self.assertTrue(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertTrue(self.app.dialog.details.isVisible())
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Continue"))
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertTrue(self.ui.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Continue"))
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
 
     def test_system_crash_layout(self):
         """Display crash dialog for a system application crash.
@@ -359,30 +357,30 @@ class T(unittest.TestCase):
         | [ Show Details ]                                   [ Continue ] |
         +-----------------------------------------------------------------+
         """
-        self.app.report["ProblemType"] = "Crash"
-        self.app.report["CrashCounter"] = "1"
-        self.app.report["Package"] = "apport 1.2.3~0ubuntu1"
+        self.ui.report["ProblemType"] = "Crash"
+        self.ui.report["CrashCounter"] = "1"
+        self.ui.report["Package"] = "apport 1.2.3~0ubuntu1"
         QTimer.singleShot(0, QCoreApplication.quit)
-        self.app.ui_present_report_details(True)
-        self.assertEqual(self.app.dialog.windowTitle(), self.distro.split()[0])
+        self.ui.ui_present_report_details(True)
+        self.assertEqual(self.ui.dialog.windowTitle(), self.distro.split()[0])
         self.assertEqual(
-            self.app.dialog.heading.text(),
+            self.ui.dialog.heading.text(),
             _("Sorry, %s has experienced an internal error.") % self.distro,
         )
         self.assertEqual(
-            self.app.dialog.text.text(),
+            self.ui.dialog.text.text(),
             _("If you notice further problems, try restarting the computer."),
         )
-        self.assertTrue(self.app.dialog.text.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertTrue(self.app.dialog.details.isVisible())
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Continue"))
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
-        self.assertTrue(self.app.dialog.ignore_future_problems.isVisible())
+        self.assertTrue(self.ui.dialog.text.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertTrue(self.ui.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Continue"))
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
+        self.assertTrue(self.ui.dialog.ignore_future_problems.isVisible())
         self.assertTrue(
-            str(self.app.dialog.ignore_future_problems.text()).endswith("of this type")
+            str(self.ui.dialog.ignore_future_problems.text()).endswith("of this type")
         )
 
     def test_apport_bug_package_layout(self):
@@ -399,22 +397,22 @@ class T(unittest.TestCase):
         | [ Cancel ]                                               [ Send ] |
         +-------------------------------------------------------------------+
         """
-        self.app.report_file = None
+        self.ui.report_file = None
         QTimer.singleShot(0, QCoreApplication.quit)
-        self.app.ui_present_report_details(True)
-        self.assertEqual(self.app.dialog.windowTitle(), self.distro.split()[0])
+        self.ui.ui_present_report_details(True)
+        self.assertEqual(self.ui.dialog.windowTitle(), self.distro.split()[0])
         self.assertEqual(
-            self.app.dialog.heading.text(), _("Send problem report to the developers?")
+            self.ui.dialog.heading.text(), _("Send problem report to the developers?")
         )
-        self.assertFalse(self.app.dialog.text.isVisible())
-        self.assertFalse(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertFalse(self.app.dialog.details.isVisible())
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Send"))
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
-        self.assertTrue(self.app.dialog.cancel_button.isVisible())
-        self.assertTrue(self.app.dialog.treeview.isVisible())
+        self.assertFalse(self.ui.dialog.text.isVisible())
+        self.assertFalse(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertFalse(self.ui.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Send"))
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
+        self.assertTrue(self.ui.dialog.cancel_button.isVisible())
+        self.assertTrue(self.ui.dialog.treeview.isVisible())
 
     def test_recoverable_crash_layout(self):
         """Display crash dialog for a recoverable crash.
@@ -428,9 +426,9 @@ class T(unittest.TestCase):
         | [ Show Details ]                                   [ Continue ] |
         +-----------------------------------------------------------------+
         """
-        self.app.report["ProblemType"] = "RecoverableProblem"
-        self.app.report["Package"] = "apport 1.2.3~0ubuntu1"
-        self.app.report["DialogBody"] = "Some developer-specified error text."
+        self.ui.report["ProblemType"] = "RecoverableProblem"
+        self.ui.report["Package"] = "apport 1.2.3~0ubuntu1"
+        self.ui.report["DialogBody"] = "Some developer-specified error text."
 
         with tempfile.NamedTemporaryFile(mode="w+") as fp:
             fp.write(
@@ -444,21 +442,21 @@ class T(unittest.TestCase):
                 )
             )
             fp.flush()
-            self.app.report["DesktopFile"] = fp.name
+            self.ui.report["DesktopFile"] = fp.name
             QTimer.singleShot(0, QCoreApplication.quit)
-            self.app.ui_present_report_details(True)
-        self.assertEqual(self.app.dialog.windowTitle(), self.distro.split()[0])
+            self.ui.ui_present_report_details(True)
+        self.assertEqual(self.ui.dialog.windowTitle(), self.distro.split()[0])
         msg = "The application Apport has experienced an internal error."
-        self.assertEqual(self.app.dialog.heading.text(), msg)
+        self.assertEqual(self.ui.dialog.heading.text(), msg)
         msg = "Some developer-specified error text."
-        self.assertEqual(self.app.dialog.text.text(), msg)
-        self.assertTrue(self.app.dialog.text.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isVisible())
-        self.assertTrue(self.app.dialog.send_error_report.isChecked())
-        self.assertTrue(self.app.dialog.details.isVisible())
-        self.assertTrue(self.app.dialog.continue_button.isVisible())
-        self.assertEqual(self.app.dialog.continue_button.text(), _("Continue"))
-        self.assertFalse(self.app.dialog.closed_button.isVisible())
+        self.assertEqual(self.ui.dialog.text.text(), msg)
+        self.assertTrue(self.ui.dialog.text.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isVisible())
+        self.assertTrue(self.ui.dialog.send_error_report.isChecked())
+        self.assertTrue(self.ui.dialog.details.isVisible())
+        self.assertTrue(self.ui.dialog.continue_button.isVisible())
+        self.assertEqual(self.ui.dialog.continue_button.text(), _("Continue"))
+        self.assertFalse(self.ui.dialog.closed_button.isVisible())
 
     def test_ui_question_choice_hide_dialog(self):
         """Test hiding/closing a UI question choice dialog.
@@ -487,35 +485,33 @@ class T(unittest.TestCase):
                 QTimer.singleShot(200, hide_dialog)  # pragma: no cover
 
             QTimer.singleShot(200, hide_dialog)
-            answer = self.app.ui_question_choice(
+            answer = self.ui.ui_question_choice(
                 "Ultimate Question", ["7", "42", "69"], False
             )
-        self.assertEqual(answer, None)
+        self.assertIsNone(answer)
 
-    @unittest.mock.patch.object(
-        MainUserInterface, "open_url", unittest.mock.MagicMock()
-    )
-    @unittest.mock.patch("apport.report.Report.add_gdb_info", unittest.mock.MagicMock())
+    @unittest.mock.patch.object(MainUserInterface, "open_url", MagicMock())
+    @unittest.mock.patch("apport.report.Report.add_gdb_info", MagicMock())
     @unittest.mock.patch(
-        "apport.fileutils.allowed_to_report", unittest.mock.MagicMock(return_value=True)
+        "apport.fileutils.allowed_to_report", MagicMock(return_value=True)
     )
     def test_1_crash_nodetails(self):
         """Crash report without showing details"""
 
         def cont():
-            if self.app.dialog and self.app.dialog.continue_button.isVisible():
-                self.app.dialog.continue_button.click()
+            if self.ui.dialog and self.ui.dialog.continue_button.isVisible():
+                self.ui.dialog.continue_button.click()
                 return
             # try again
             QTimer.singleShot(1000, cont)  # pragma: no cover
 
         QTimer.singleShot(1000, cont)
         with wrap_object(apport_kde.ProgressDialog, "__init__") as progress_dialog:
-            self.app.run_crash(self.app.report_file)
+            self.ui.run_crash(self.ui.report_file)
 
         # we should have reported one crash
-        self.assertEqual(self.app.crashdb.latest_id(), 0)
-        r = self.app.crashdb.download(0)
+        self.assertEqual(self.ui.crashdb.latest_id(), 0)
+        r = self.ui.crashdb.download(0)
         self.assertEqual(r["ProblemType"], "Crash")
         self.assertEqual(r["ExecutablePath"], "/bin/bash")
 
@@ -529,47 +525,46 @@ class T(unittest.TestCase):
         self.assertIn("libc", r["Dependencies"])
 
         # URL was opened
-        self.assertEqual(self.app.open_url.call_count, 1)
+        self.assertEqual(self.ui.open_url.call_count, 1)
 
-    @unittest.mock.patch.object(
-        MainUserInterface, "open_url", unittest.mock.MagicMock()
-    )
-    @unittest.mock.patch("apport.report.Report.add_gdb_info", unittest.mock.MagicMock())
+    @unittest.mock.patch.object(MainUserInterface, "open_url", MagicMock())
+    @unittest.mock.patch("apport.report.Report.add_gdb_info", MagicMock())
     @unittest.mock.patch(
-        "apport.fileutils.allowed_to_report", unittest.mock.MagicMock(return_value=True)
+        "apport.fileutils.allowed_to_report", MagicMock(return_value=True)
     )
     def test_1_crash_details(self):
         """Crash report with showing details"""
 
         def show_details():
-            if self.app.dialog and self.app.dialog.show_details.isVisible():
-                self.app.dialog.show_details.click()
+            if self.ui.dialog and self.ui.dialog.show_details.isVisible():
+                self.ui.dialog.show_details.click()
                 QTimer.singleShot(1000, cont)
                 return
 
             # try again
             QTimer.singleShot(200, show_details)  # pragma: no cover
 
-        def cont():
+        def cont() -> None:
             # wait until data collection is done and tree filled
-            details = self.app.dialog.findChild(QTreeWidget, "details")
+            details = self.ui.dialog.findChild(QTreeWidget, "details")
+            assert isinstance(details, QTreeWidget)
             if details.topLevelItemCount() == 0:
                 QTimer.singleShot(200, cont)
                 return
 
-            if self.app.dialog and self.app.dialog.continue_button.isVisible():
-                self.app.dialog.continue_button.click()
+            if self.ui.dialog and self.ui.dialog.continue_button.isVisible():
+                self.ui.dialog.continue_button.click()
                 return
             # try again
             QTimer.singleShot(200, cont)  # pragma: no cover
 
         QTimer.singleShot(200, show_details)
         with wrap_object(apport_kde.ProgressDialog, "__init__") as progress_dialog:
-            self.app.run_crash(self.app.report_file)
+            self.ui.run_crash(self.ui.report_file)
 
         # we should have reported one crash
-        self.assertEqual(self.app.crashdb.latest_id(), 0)
-        r = self.app.crashdb.download(0)
+        self.assertEqual(self.ui.crashdb.latest_id(), 0)
+        r = self.ui.crashdb.download(0)
         self.assertEqual(r["ProblemType"], "Crash")
         self.assertEqual(r["ExecutablePath"], "/bin/bash")
 
@@ -581,39 +576,37 @@ class T(unittest.TestCase):
         self.assertIn("libc", r["Dependencies"])
 
         # URL was opened
-        self.assertEqual(self.app.open_url.call_count, 1)
+        self.assertEqual(self.ui.open_url.call_count, 1)
 
-    @unittest.mock.patch.object(
-        MainUserInterface, "open_url", unittest.mock.MagicMock()
-    )
-    @unittest.mock.patch("apport.report.Report.add_gdb_info", unittest.mock.MagicMock())
+    @unittest.mock.patch.object(MainUserInterface, "open_url", MagicMock())
+    @unittest.mock.patch("apport.report.Report.add_gdb_info", MagicMock())
     @unittest.mock.patch(
-        "apport.fileutils.allowed_to_report", unittest.mock.MagicMock(return_value=True)
+        "apport.fileutils.allowed_to_report", MagicMock(return_value=True)
     )
     def test_1_crash_noaccept(self):
         """Crash report with non-accepting crash DB"""
 
         def cont():
-            if self.app.dialog and self.app.dialog.continue_button.isVisible():
-                self.app.dialog.continue_button.click()
+            if self.ui.dialog and self.ui.dialog.continue_button.isVisible():
+                self.ui.dialog.continue_button.click()
                 return
             # try again
             QTimer.singleShot(1000, cont)  # pragma: no cover
 
         QTimer.singleShot(1000, cont)
-        self.app.crashdb.options["problem_types"] = ["bug"]
+        self.ui.crashdb.options["problem_types"] = ["bug"]
         with wrap_object(apport_kde.ProgressDialog, "__init__") as progress_dialog:
-            self.app.run_crash(self.app.report_file)
+            self.ui.run_crash(self.ui.report_file)
 
         # we should not have reported the crash
-        self.assertEqual(self.app.crashdb.latest_id(), -1)
-        self.assertEqual(self.app.open_url.call_count, 0)
+        self.assertEqual(self.ui.crashdb.latest_id(), -1)
+        self.assertEqual(self.ui.open_url.call_count, 0)
 
         # no progress dialog for non-accepting DB
         progress_dialog.assert_not_called()
 
         # data was collected for whoopsie
-        r = self.app.report
+        r = self.ui.report
         self.assertEqual(r["ProblemType"], "Crash")
         self.assertEqual(r["ExecutablePath"], "/bin/bash")
         self.assertTrue(r["Package"].startswith("bash "))
@@ -621,93 +614,89 @@ class T(unittest.TestCase):
 
     def test_bug_report_installed_package(self):
         """Bug report for installed package."""
-        self.app.report_file = None
-        self.app.args.package = "bash"
+        self.ui.report_file = None
+        self.ui.args.package = "bash"
 
         def c():
-            if self.app.dialog and self.app.dialog.cancel_button.isVisible():
-                self.app.dialog.cancel_button.click()
+            if self.ui.dialog and self.ui.dialog.cancel_button.isVisible():
+                self.ui.dialog.cancel_button.click()
                 return
             # try again
             QTimer.singleShot(1000, c)  # pragma: no cover
 
         QTimer.singleShot(1000, c)
-        self.app.run_report_bug()
+        self.ui.run_report_bug()
 
-        self.assertEqual(self.app.report["ProblemType"], "Bug")
-        self.assertEqual(self.app.report["SourcePackage"], "bash")
-        self.assertTrue(self.app.report["Package"].startswith("bash "))
-        self.assertNotEqual(self.app.report["Dependencies"], "")
+        self.assertEqual(self.ui.report["ProblemType"], "Bug")
+        self.assertEqual(self.ui.report["SourcePackage"], "bash")
+        self.assertTrue(self.ui.report["Package"].startswith("bash "))
+        self.assertNotEqual(self.ui.report["Dependencies"], "")
 
     def test_bug_report_uninstalled_package(self):
         """Bug report for uninstalled package"""
         pkg = apport.packaging.get_uninstalled_package()
 
-        self.app.report_file = None
-        self.app.args.package = pkg
+        self.ui.report_file = None
+        self.ui.args.package = pkg
 
         def c():
-            if self.app.dialog and self.app.dialog.cancel_button.isVisible():
-                self.app.dialog.cancel_button.click()
+            if self.ui.dialog and self.ui.dialog.cancel_button.isVisible():
+                self.ui.dialog.cancel_button.click()
                 return
             # try again
             QTimer.singleShot(1000, c)  # pragma: no cover
 
         QTimer.singleShot(1000, c)
-        self.app.run_report_bug()
+        self.ui.run_report_bug()
 
-        self.assertEqual(self.app.report["ProblemType"], "Bug")
+        self.assertEqual(self.ui.report["ProblemType"], "Bug")
         self.assertEqual(
-            self.app.report["SourcePackage"], apport.packaging.get_source(pkg)
+            self.ui.report["SourcePackage"], apport.packaging.get_source(pkg)
         )
-        self.assertEqual(self.app.report["Package"], f"{pkg} (not installed)")
+        self.assertEqual(self.ui.report["Package"], f"{pkg} (not installed)")
 
-    @unittest.mock.patch.object(
-        MainUserInterface, "open_url", unittest.mock.MagicMock()
-    )
+    @unittest.mock.patch.object(MainUserInterface, "open_url", MagicMock())
     def test_1_update_report(self):
         """Updating an existing report"""
-        self.app.report_file = None
+        self.ui.report_file = None
 
         def cont():
-            if self.app.dialog and self.app.dialog.continue_button.isVisible():
-                self.app.dialog.continue_button.click()
+            if self.ui.dialog and self.ui.dialog.continue_button.isVisible():
+                self.ui.dialog.continue_button.click()
                 return
             # try again
             QTimer.singleShot(200, cont)  # pragma: no cover
 
         # upload empty report
-        crash_id = self.app.crashdb.upload({})
+        crash_id = self.ui.crashdb.upload({})
         self.assertEqual(crash_id, 0)
-        self.app.args.update_report = 0
-        self.app.args.package = "bash"
+        self.ui.args.update_report = 0
+        self.ui.args.package = "bash"
 
         QTimer.singleShot(200, cont)
-        self.app.run_update_report()
+        self.ui.run_update_report()
 
         # no new bug reported
-        self.assertEqual(self.app.crashdb.latest_id(), 0)
+        self.assertEqual(self.ui.crashdb.latest_id(), 0)
 
         # bug was updated
-        r = self.app.crashdb.download(0)
+        r = self.ui.crashdb.download(0)
         self.assertTrue(r["Package"].startswith("bash "))
         self.assertIn("libc", r["Dependencies"])
         self.assertIn("DistroRelease", r)
 
         # No URL in this mode
-        self.assertEqual(self.app.open_url.call_count, 0)
+        self.assertEqual(self.ui.open_url.call_count, 0)
 
-    @unittest.mock.patch.object(
-        MainUserInterface, "open_url", unittest.mock.MagicMock()
-    )
+    @unittest.mock.patch.object(MainUserInterface, "open_url", MagicMock())
     def test_1_update_report_different_binary_source(self):
         """Updating an existing report on a source package which does not have
         a binary of the same name"""
-        self.app.report_file = None
+        self.ui.report_file = None
 
         def cont():
-            if self.app.dialog and self.app.dialog.continue_button.isVisible():
-                self.app.dialog.continue_button.click()
+            if self.ui.dialog and self.ui.dialog.continue_button.isVisible():
+                self.ui.dialog.continue_button.click()
                 return
             # try again
             QTimer.singleShot(200, cont)  # pragma: no cover
@@ -726,40 +715,60 @@ class T(unittest.TestCase):
             f.write('def add_info(r, ui):\n r["MachineType"]="Laptop"\n')
 
         # upload empty report
-        crash_id = self.app.crashdb.upload({})
+        crash_id = self.ui.crashdb.upload({})
         self.assertEqual(crash_id, 0)
 
         # run in update mode for that bug
-        self.app.args.update_report = 0
-        self.app.args.package = source_pkg
+        self.ui.args.update_report = 0
+        self.ui.args.package = source_pkg
 
         QTimer.singleShot(200, cont)
-        self.app.run_update_report()
+        self.ui.run_update_report()
 
         # no new bug reported
-        self.assertEqual(self.app.crashdb.latest_id(), 0)
+        self.assertEqual(self.ui.crashdb.latest_id(), 0)
 
         # bug was updated
-        r = self.app.crashdb.download(0)
+        r = self.ui.crashdb.download(0)
         self.assertIn("ProcEnviron", r)
         self.assertIn("DistroRelease", r)
         self.assertIn("Uname", r)
         self.assertEqual(r["MachineType"], "Laptop")
 
         # No URL in this mode
-        self.assertEqual(self.app.open_url.call_count, 0)
+        self.assertEqual(self.ui.open_url.call_count, 0)
 
     def test_administrator_disabled_reporting(self):
         QTimer.singleShot(0, QCoreApplication.quit)
-        self.app.ui_present_report_details(False)
-        self.assertFalse(self.app.dialog.send_error_report.isVisible())
-        self.assertFalse(self.app.dialog.send_error_report.isChecked())
+        self.ui.ui_present_report_details(False)
+        self.assertFalse(self.ui.dialog.send_error_report.isVisible())
+        self.assertFalse(self.ui.dialog.send_error_report.isChecked())
 
-    def test_ui_set_upload_progress(self):
-        self.app.ui_start_upload_progress()
+    def test_ui_run_terminal(self) -> None:
+        """Test ui_run_terminal."""
+        if not self.ui.ui_has_terminal():
+            self.skipTest("installed terminal application needed")
+        self.ui.ui_run_terminal("true")
+
+    def test_ui_set_upload_progress(self) -> None:
+        self.ui.ui_start_upload_progress()
         try:
-            self.app.ui_set_upload_progress(0.5)
-            progress = self.app.progress.findChild(QProgressBar, "progress")
+            self.ui.ui_set_upload_progress(0.5)
+            progress = self.ui.progress.findChild(QProgressBar, "progress")
+            assert isinstance(progress, QProgressBar)
             self.assertEqual(progress.value(), 500)
         finally:
-            self.app.ui_stop_upload_progress()
+            self.ui.ui_stop_upload_progress()
+
+    def test_ui_question_file_close(self) -> None:
+        def cont() -> None:
+            for widget in self.ui.app.allWidgets():
+                if isinstance(widget, QFileDialog):
+                    widget.close()
+                    return
+
+            # try again
+            QTimer.singleShot(10, cont)  # pragma: no cover
+
+        QTimer.singleShot(10, cont)
+        self.assertIsNone(self.ui.ui_question_file("Please select a file"))
