@@ -1,5 +1,7 @@
 """Test test helper functions. Test inception for the win!"""
 
+# pylint: disable=missing-class-docstring,missing-function-docstring
+
 import unittest
 
 from tests.helper import get_init_system, wrap_object
@@ -11,11 +13,11 @@ class Multiply:  # pylint: disable=too-few-public-methods
     def __init__(self, multiplier):
         self.multiplier = multiplier
 
-    def multiply(self, x: int) -> int:
-        return x * self.multiplier
+    def multiply(self, factor: int) -> int:
+        return factor * self.multiplier
 
 
-class T(unittest.TestCase):
+class TestTestHelper(unittest.TestCase):
     def test_get_init_systemd(self):
         open_mock = unittest.mock.mock_open(read_data="systemd\n")
         with unittest.mock.patch("builtins.open", open_mock):
@@ -24,6 +26,6 @@ class T(unittest.TestCase):
 
     def test_wrap_object_with_statement(self):
         with wrap_object(Multiply, "__init__") as mock:
-            m = Multiply(7)
-            self.assertEqual(m.multiply(6), 42)
+            multiply = Multiply(7)
+            self.assertEqual(multiply.multiply(6), 42)
         mock.assert_called_once_with(7)
