@@ -3,12 +3,23 @@
 # TODO: Address following pylint complaints
 # pylint: disable=invalid-name
 
+import base64
+import contextlib
+import datetime
 import email
 import io
 import locale
+import sys
 import textwrap
 import time
 import unittest
+import unittest.mock
+from unittest.mock import MagicMock
+
+try:
+    import zstandard
+except ImportError:
+    zstandard = None  # type: ignore
 
 import problem_report
 
@@ -74,20 +85,22 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
 
     def test_get_timestamp_locale_german(self):
         """get_timestamp() returns date when LC_TIME is set."""
-        pr = problem_report.ProblemReport(date="Wed May 18 09:49:57 2022")
+        now = datetime.datetime.now()
+
+        pr = problem_report.ProblemReport(date=now.strftime("%a %b %d %H:%M:%S %Y"))
         orig_ctime = locale.getlocale(locale.LC_TIME)
         try:
             locale.setlocale(locale.LC_TIME, "de_DE.UTF-8")
         except locale.Error:
             self.skipTest("Missing German locale support")
-        self.assertEqual(pr.get_timestamp(), 1652867397 + time.altzone)
+        self.assertEqual(pr.get_timestamp(), int(now.timestamp()))
         locale.setlocale(locale.LC_TIME, orig_ctime)
 
     def test_get_timestamp_returns_none(self):
         """get_timestamp() returns None."""
         pr = problem_report.ProblemReport()
         del pr["Date"]
-        self.assertEqual(pr.get_timestamp(), None)
+        self.assertIsNone(pr.get_timestamp())
 
     def test_consistency_checks(self):
         """Various error conditions."""
@@ -105,7 +118,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertRaises(TypeError, pr.__setitem__, "a", ["/tmp/nonexistent"])
         self.assertRaises(KeyError, pr.__getitem__, "Nonexistent")
 
-    def test_write(self):
+    def test_write(self) -> None:
         """write() and proper formatting."""
         pr = problem_report.ProblemReport(date="now!")
         pr["Simple"] = "bar"
@@ -137,8 +150,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
                 """
             )
             + "  foo   bar\n baz\n   blip  \n \n afteremptyline\n"
-        )
-        expected = expected.encode("UTF-8")
+        ).encode("UTF-8")
         self.assertEqual(out.getvalue(), expected)
 
     def test_load(self):
@@ -272,6 +284,88 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
             ):
                 report.load(report_file)
 
+    @unittest.skipUnless(zstandard, "zstandard Python module not available")
+    def test_load_zstd_compressed_data(self) -> None:
+        """Test reading zstd-compressed data."""
+        report = problem_report.ProblemReport()
+        content = (
+            b"CoreDump: base64\n"
+            b" KLUv/SROHQIAgoQOEqC7ASCSGCybZRrN7//Hsn7dVyActu7bbMcLaav0RC06\n"
+            b" m6fCZ/N7aeOeyqxspRVn88bSx8a8opWQAwEA/8OEAhOOwLA=\n"
+        )
+        with io.BytesIO(content) as report_file:
+            report.load(report_file)
+        self.assertEqual(
+            report["CoreDump"],
+            "sample data that is compressed with zstd"
+            " and is long enough for zstd to work.\n",
+        )
+
+    @unittest.skipUnless(zstandard, "zstandard Python module not available")
+    def test_reading_zstd_compressed_value(self) -> None:
+        """Test reading zstd-compressed CompressedValue."""
+        report = problem_report.ProblemReport()
+        content = (
+            b"CoreDump: base64\n"
+            b" KLUv/SRPHQIA8sQPFKClOoBIEkjIbmfbhiz9/1/96mYFATGiz9JHLfTZY9aV\n"
+            b" a9QBRdQz0AkUjbdbVqNQorP3UpGC1NIi9X3UgygBAEyQcYs=\n"
+        )
+        with io.BytesIO(content) as report_file:
+            report.load(report_file, binary="compressed")
+        self.assertEqual(
+            report["CoreDump"].get_value(),
+            b"fake core dump data for testing purposes"
+            b" which is long enough to be compressed\n",
+        )
+
+    @unittest.skipUnless(zstandard, "zstandard Python module not available")
+    def test_writing_zstd_compressed_value(self) -> None:
+        """Test writing zstd-compressed CompressedValue."""
+        compressed_value = problem_report.CompressedValue(
+            compressed_value=base64.b64decode(
+                b"KLUv/SRCvQEAIsQMEMC3AbLcS0WaZ8rvf69jyw3ghpXr6pBr5i7HiCKroc60"
+                b"3+tB4rC/4TV1osHyJUXme6IIAD6PTiM="
+            )
+        )
+        output = io.BytesIO()
+        compressed_value.write(output)
+        self.assertEqual(
+            output.getvalue(),
+            b"Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam\n",
+        )
+
+    @unittest.skipUnless(zstandard, "zstandard Python module not available")
+    def test_zstd_compressed_value_length(self) -> None:
+        """Test getting length of zstd-compressed CompressedValue."""
+        compressed_value = problem_report.CompressedValue(
+            compressed_value=base64.b64decode(
+                b"KLUv/SRB3QEAwoMMEbDrOJRilklIQlLIJC3OvFUMIIwdaHxrV6aSHEv6rFlP"
+                b"/ll/IaXfo19actUkdvm8hggBAAlhlgHbiBMF"
+            )
+        )
+        self.assertEqual(len(compressed_value), 65)
+
+    @unittest.mock.patch("builtins.__import__")
+    def test_zstandard_missing(self, import_mock: MagicMock) -> None:
+        """Test reading zstd-compressed data when zstandard is missing."""
+        with contextlib.suppress(KeyError):
+            sys.modules.pop("zstandard")
+        import_mock.side_effect = ImportError("mocked import error")
+
+        report = problem_report.ProblemReport()
+        content = (
+            b"CoreDump: base64\n"
+            b" KLUv/SROHQIAgoQOEqC7ASCSGCybZRrN7//Hsn7dVyActu7bbMcLaav0RC06\n"
+            b" m6fCZ/N7aeOeyqxspRVn88bSx8a8opWQAwEA/8OEAhOOwLA=\n"
+        )
+        expected_message = (
+            "Failed to import zstandard library: mocked import error."
+            " Please install python3-zstandard."
+        )
+        with self.assertRaisesRegex(RuntimeError, expected_message):
+            with io.BytesIO(content) as report_file:
+                report.load(report_file)
+
     def test_write_fileobj(self):
         """Write a report with a pointer to a file-like object."""
         tempbin = io.BytesIO(bin_data)
@@ -326,7 +420,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
 
         # test with skipping binary data
         pr.load(io.BytesIO(bin_report), binary=False)
-        self.assertEqual(pr["File"], None)
+        self.assertIsNone(pr["File"])
         self.assertEqual(pr.has_removed_fields(), True)
 
         # test with keeping compressed binary data
@@ -336,6 +430,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertTrue(isinstance(pr["File"], problem_report.CompressedValue))
         self.assertEqual(len(pr["File"]), len(bin_data))
         self.assertEqual(pr["File"].get_value(), bin_data)
+        self.assertEqual(pr["File"].name, "File")
 
     def test_read_file_legacy(self):
         """Read a report with binary data in legacy format without gzip
@@ -359,7 +454,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
 
         # test with skipping binary data
         pr.load(io.BytesIO(bin_report), binary=False)
-        self.assertEqual(pr["File"], None)
+        self.assertIsNone(pr["File"])
         self.assertEqual(pr.has_removed_fields(), True)
 
         # test with keeping CompressedValues
@@ -367,6 +462,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertEqual(pr.has_removed_fields(), False)
         self.assertEqual(len(pr["File"]), 31)
         self.assertEqual(pr["File"].get_value(), b"AB" * 10 + b"\0" * 10 + b"Z")
+        self.assertEqual(pr["File"].name, "File")
         out = io.BytesIO()
         pr["File"].write(out)
         out.seek(0)
@@ -433,6 +529,35 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
             ).encode(),
         )
 
+    def test_sorted_items(self) -> None:
+        """Test ProblemReport.sorted_items()."""
+        bin_report = textwrap.dedent(
+            """\
+            ProblemType: Crash
+            Date: now!
+            File: base64
+             eJw=
+             c3RyxIAMcBAFAG55BXk=
+            _MarkForUpload: False
+            Architecture: amd64
+            ExecutablePath: /usr/bin/python3
+            Package: python3.12-minimal
+            """
+        ).encode()
+
+        report = problem_report.ProblemReport()
+        report.load(io.BytesIO(bin_report), binary=False)
+        self.assertEqual(
+            list(report.sorted_items()),
+            [
+                ("ExecutablePath", "/usr/bin/python3"),
+                ("Package", "python3.12-minimal"),
+                ("ProblemType", "Crash"),
+                ("Architecture", "amd64"),
+                ("Date", "now!"),
+            ],
+        )
+
     def test_write_mime_text(self):
         """write_mime() for text values."""
         pr = problem_report.ProblemReport(date="now!")
@@ -469,7 +594,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertTrue(not parts[1].is_multipart())
         self.assertEqual(parts[1].get_content_type(), "text/plain")
         self.assertEqual(parts[1].get_content_charset(), "utf-8")
-        self.assertEqual(parts[1].get_filename(), None)
+        self.assertIsNone(parts[1].get_filename())
         expected = textwrap.dedent(
             f"""\
             ProblemType: Crash
@@ -590,7 +715,7 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertTrue(not parts[1].is_multipart())
         self.assertEqual(parts[1].get_content_type(), "text/plain")
         self.assertEqual(parts[1].get_content_charset(), "utf-8")
-        self.assertEqual(parts[1].get_filename(), None)
+        self.assertIsNone(parts[1].get_filename())
         self.assertEqual(
             parts[1].get_payload(decode=True),
             textwrap.dedent(
@@ -639,10 +764,11 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
         pr["oldbin"] = bin_data
         pr["overwrite"] = "I am crap"
 
-        d = {}
-        d["newtext"] = "Goodbye world"
-        d["newbin"] = "11\000\001\002\xFFZZ"
-        d["overwrite"] = "I am good"
+        d = {
+            "newtext": "Goodbye world",
+            "newbin": "11\000\001\002\xFFZZ",
+            "overwrite": "I am good",
+        }
 
         pr.update(d)
         self.assertEqual(pr["oldtext"], "Hello world")
@@ -671,3 +797,81 @@ class T(unittest.TestCase):  # pylint: disable=too-many-public-methods
         self.assertEqual(pr["DataYes"], "yesyes")
         self.assertEqual(pr["GoodFile"], bin_data)
         self.assertEqual(sorted(pr.keys()), ["DataYes", "GoodFile"])
+
+    def test_get_on_disk_size(self) -> None:
+        """Test CompressedValue.get_on_disk_size()."""
+        compressed_value = problem_report.CompressedValue(b"somedata")
+        self.assertEqual(compressed_value.get_compressed_size(), 28)
+        assert compressed_value.compressed_value is not None
+        base64_encoded = base64.b64encode(compressed_value.compressed_value)
+        self.assertEqual(compressed_value.get_on_disk_size(), len(base64_encoded))
+
+
+class TestEntryParser(unittest.TestCase):
+    # pylint: disable=protected-access
+    """Test _EntryParser class."""
+
+    def test_parse(self) -> None:
+        """Test parsing a report file with _EntryParser."""
+        report_file = io.BytesIO(
+            textwrap.dedent(
+                """\
+                First: single line
+                Second: multi
+                 line
+                """
+            ).encode()
+        )
+        entries = []
+        for entry in problem_report._EntryParser(report_file):
+            entries.append(list(entry))
+        self.assertEqual(
+            entries, [[b"First: single line\n"], [b"Second: multi\n", b" line\n"]]
+        )
+
+    def test_skip_entries(self) -> None:
+        """Test skipping reading one entry."""
+        report_file = io.BytesIO(
+            textwrap.dedent(
+                """\
+                First: this entry
+                 will be skipped
+                Second: single line
+                """
+            ).encode()
+        )
+        iterator = problem_report._EntryParser(report_file)
+        next(iterator)
+        self.assertEqual(list(next(iterator)), [b"Second: single line\n"])
+
+    def test_skip_partial_entries(self) -> None:
+        """Test skipping reading one entry."""
+        report_file = io.BytesIO(
+            textwrap.dedent(
+                """\
+                First: this line is read,
+                 but these lines
+                 will be skipped
+                Second: single line
+                """
+            ).encode()
+        )
+        iterator = problem_report._EntryParser(report_file)
+        self.assertEqual(next(next(iterator)), b"First: this line is read,\n")
+        self.assertEqual(list(next(iterator)), [b"Second: single line\n"])
+
+    def test_skip_last_entry(self) -> None:
+        """Test skipping reading the last entry."""
+        report_file = io.BytesIO(
+            textwrap.dedent(
+                """\
+                First: this line is read
+                Second: this line is skipped
+                """
+            ).encode()
+        )
+        iterator = problem_report._EntryParser(report_file)
+        self.assertEqual(list(next(iterator)), [b"First: this line is read\n"])
+        next(iterator)
+        with self.assertRaises(StopIteration):
+            next(iterator)
