@@ -9,15 +9,18 @@
 
 """Unit tests for data/apport."""
 
+import datetime
 import errno
 import io
 import os
 import pathlib
 import shutil
+import signal
 import sys
 import tempfile
 import time
 import unittest
+from unittest.mock import MagicMock
 
 import apport.fileutils
 import apport.user_group
@@ -28,6 +31,7 @@ apport_binary = import_module_from_file(get_data_directory() / "apport")
 
 
 class TestApport(unittest.TestCase):
+    # pylint: disable=too-many-public-methods
     """Unit tests for data/apport."""
 
     @classmethod
@@ -65,7 +69,7 @@ class TestApport(unittest.TestCase):
             del os.environ["APPORT_LOCK_FILE"]
 
     @unittest.mock.patch("fcntl.lockf")
-    def test_check_lock_taken(self, lockf_mock: unittest.mock.MagicMock) -> None:
+    def test_check_lock_taken(self, lockf_mock: MagicMock) -> None:
         """Test check_lock() with lock file taken by other process."""
         # Since this is a unit test, let the mock raise an exception instead
         # of letting the fcntl.lockf call run into the timeout signal.
@@ -86,7 +90,7 @@ class TestApport(unittest.TestCase):
         self.assertEqual(apport_binary.refine_core_ulimit(options), -1)
 
     @unittest.mock.patch("os.isatty")
-    def test_init_error_log_is_tty(self, isatty_mock: unittest.mock.MagicMock) -> None:
+    def test_init_error_log_is_tty(self, isatty_mock: MagicMock) -> None:
         """Test init_error_log() doing nothing on a TTY."""
         isatty_mock.return_value = True
         stderr = sys.stderr
@@ -95,13 +99,21 @@ class TestApport(unittest.TestCase):
         self.assertEqual(sys.stderr, stderr)
         isatty_mock.assert_called_once_with(2)
 
-    @unittest.mock.patch("builtins.__import__")
-    def test_receive_arguments_via_socket_import_error(self, import_mock):
+    def test_receive_arguments_via_socket_import_error(self) -> None:
         """Test receive_arguments_via_socket() fail to import systemd."""
-        import_mock.side_effect = ModuleNotFoundError("No module named 'systemd'")
-        with self.assertRaisesRegex(SystemExit, "^0$"):
+        with (
+            self.assertLogs(level="ERROR") as error_logs,
+            unittest.mock.patch("builtins.__import__") as import_mock,
+            self.assertRaisesRegex(SystemExit, "^0$"),
+        ):
+            import_mock.side_effect = ModuleNotFoundError("No module named 'systemd'")
             apport_binary.receive_arguments_via_socket()
 
+        import_mock.assert_called_once()
+        self.assertRegex(
+            error_logs.output[0], "apport-forward.socket.*systemd python module"
+        )
+
     def test_receive_arguments_via_socket_invalid_socket(self):
         """Test receive_arguments_via_socket with invalid socket."""
         try:
@@ -114,11 +126,9 @@ class TestApport(unittest.TestCase):
         with self.assertRaisesRegex(SystemExit, "^1$"):
             apport_binary.receive_arguments_via_socket()
 
+    @unittest.mock.patch.object(apport_binary, "init_error_log", MagicMock())
     @unittest.mock.patch.object(
-        apport_binary, "init_error_log", unittest.mock.MagicMock()
-    )
-    @unittest.mock.patch.object(
-        apport_binary, "is_same_ns", unittest.mock.MagicMock(return_value=False)
+        apport_binary, "is_same_ns", MagicMock(return_value=False)
     )
     @unittest.mock.patch.object(apport_binary, "forward_crash_to_container")
     def test_main_forward_crash_to_container(self, forward_mock):
@@ -127,18 +137,14 @@ class TestApport(unittest.TestCase):
         self.assertEqual(apport_binary.main(args), 0)
         forward_mock.assert_called_once()
 
-    @unittest.mock.patch.object(
-        apport_binary, "init_error_log", unittest.mock.MagicMock()
-    )
+    @unittest.mock.patch.object(apport_binary, "init_error_log", MagicMock())
     @unittest.mock.patch.object(apport_binary, "start_apport")
     def test_main_start(self, start_mock):
         """Test calling apport with --start."""
         self.assertEqual(apport_binary.main(["--start"]), 0)
         start_mock.assert_called_once_with()
 
-    @unittest.mock.patch.object(
-        apport_binary, "init_error_log", unittest.mock.MagicMock()
-    )
+    @unittest.mock.patch.object(apport_binary, "init_error_log", MagicMock())
     @unittest.mock.patch.object(apport_binary, "stop_apport")
     def test_main_stop(self, stop_mock):
         """Test calling apport with --stop."""
@@ -195,14 +201,12 @@ class TestApport(unittest.TestCase):
         )
         self.assertEqual(open_mock.call_count, 3)
 
-    @unittest.mock.patch("os.setresgid", unittest.mock.MagicMock())
-    @unittest.mock.patch("os.setresuid", unittest.mock.MagicMock())
+    @unittest.mock.patch("os.setresgid", MagicMock())
+    @unittest.mock.patch("os.setresuid", MagicMock())
     @unittest.mock.patch.object(apport_binary, "_run_with_output_limit_and_timeout")
     @unittest.mock.patch("os.path.exists")
     def test_is_closing_session(
-        self,
-        path_exist_mock: unittest.mock.MagicMock,
-        run_mock: unittest.mock.MagicMock,
+        self, path_exist_mock: MagicMock, run_mock: MagicMock
     ) -> None:
         """Test is_closing_session()."""
         path_exist_mock.return_value = True
@@ -252,14 +256,12 @@ class TestApport(unittest.TestCase):
                     apport_binary.is_closing_session(proc_pid, crash_user), False
                 )
 
-    @unittest.mock.patch("os.setresgid", unittest.mock.MagicMock())
-    @unittest.mock.patch("os.setresuid", unittest.mock.MagicMock())
+    @unittest.mock.patch("os.setresgid", MagicMock())
+    @unittest.mock.patch("os.setresuid", MagicMock())
     @unittest.mock.patch.object(apport_binary, "_run_with_output_limit_and_timeout")
     @unittest.mock.patch("os.path.exists")
     def test_is_closing_session_gdbus_failure(
-        self,
-        path_exist_mock: unittest.mock.MagicMock,
-        run_mock: unittest.mock.MagicMock,
+        self, path_exist_mock: MagicMock, run_mock: MagicMock
     ) -> None:
         """Test is_closing_session() with OSError from gdbus."""
         path_exist_mock.return_value = True
@@ -277,10 +279,8 @@ class TestApport(unittest.TestCase):
         path_exist_mock.assert_called_once_with("/run/user/1337/bus")
         run_mock.assert_called_once()
 
-    @unittest.mock.patch.object(apport_binary, "check_lock", unittest.mock.MagicMock())
-    @unittest.mock.patch.object(
-        apport_binary, "init_error_log", unittest.mock.MagicMock()
-    )
+    @unittest.mock.patch.object(apport_binary, "check_lock", MagicMock())
+    @unittest.mock.patch.object(apport_binary, "init_error_log", MagicMock())
     def test_missing_proc_pid(self) -> None:
         """Test /proc/<pid> is already gone."""
         fake_pid = 2147483647
@@ -291,3 +291,117 @@ class TestApport(unittest.TestCase):
                 with unittest.mock.patch("sys.stdin", return_value=stdin_mock):
                     self.assertEqual(apport_binary.main(["-p", str(fake_pid)]), 1)
         self.assertIn("/proc/2147483647 not found", error_logs.output[0])
+
+    @unittest.mock.patch.object(apport_binary, "check_lock", MagicMock())
+    @unittest.mock.patch.object(apport_binary, "consistency_checks", MagicMock())
+    @unittest.mock.patch.object(apport_binary, "init_error_log", MagicMock())
+    @unittest.mock.patch.object(
+        apport_binary, "is_closing_session", MagicMock(return_value=False)
+    )
+    @unittest.mock.patch.object(apport_binary, "is_systemd_watchdog_restart")
+    def test_main_ignore_watchdog_restart(
+        self, is_systemd_watchdog_restart_mock: MagicMock
+    ) -> None:
+        """Test main() to ignore watchdog restarts.
+
+        The underlying is_systemd_watchdog_restart() function is mocked,
+        which should be tested in a separate unit test.
+        """
+        is_systemd_watchdog_restart_mock.return_value = True
+        args = ["-p", str(os.getpid()), "-s", str(int(signal.SIGABRT)), "-c", "-1"]
+        with self.assertLogs(level="ERROR") as error_logs:
+            with tempfile.NamedTemporaryFile() as stdin:
+                stdin_mock = io.TextIOWrapper(io.FileIO(stdin.name))
+                with unittest.mock.patch("sys.stdin", stdin_mock):
+                    exit_code = apport_binary.main(args)
+
+        self.assertEqual(exit_code, 0)
+        is_systemd_watchdog_restart_mock.assert_called_once()
+        self.assertIn("Ignoring systemd watchdog restart", error_logs.output[0])
+
+    @unittest.mock.patch.object(apport_binary, "init_error_log", MagicMock())
+    def test_non_existing_systemd_coredump(self) -> None:
+        """Test main() to print error if systemd-coredump cannot be found."""
+        try:
+            # pylint: disable-next=import-outside-toplevel
+            import systemd.journal
+        except ImportError as error:
+            self.skipTest(f"{error.name} Python module not available")
+        assert systemd.journal
+
+        with self.assertLogs(level="ERROR") as error_logs:
+            exit_code = apport_binary.main(["--from-systemd-coredump", "42-5846584-0"])
+
+        self.assertEqual(exit_code, 1)
+        self.assertIn(
+            "No journal log for systemd unit"
+            " systemd-coredump@42-5846584-0.service found.",
+            error_logs.output[0],
+        )
+
+    def test_systemd_journal_import_error(self) -> None:
+        """Test handling missing systemd.journal library correctly."""
+        with (
+            self.assertLogs(level="ERROR") as error_logs,
+            unittest.mock.patch("builtins.__import__") as import_mock,
+            self.assertRaisesRegex(SystemExit, "^1$"),
+        ):
+            import_mock.side_effect = ModuleNotFoundError(
+                "No module named 'systemd.journal'"
+            )
+            apport_binary.process_crash_from_systemd_coredump("0-383264-0")
+
+        import_mock.assert_called_once()
+        self.assertRegex(error_logs.output[0], "Please install python3-systemd")
+
+    @unittest.mock.patch.object(apport_binary, "process_crash")
+    @unittest.mock.patch.object(apport_binary, "get_systemd_coredump")
+    def test_reading_core_from_journal_log(
+        self, get_systemd_coredump_mock: MagicMock, process_crash_mock: MagicMock
+    ) -> None:
+        """Test _user_can_read_coredump via process_crash_from_systemd_coredump
+
+        The core dump can be provided via the journal log. In this case
+        _user_can_read_coredump should behave correctly.
+        """
+        get_systemd_coredump_mock.return_value = {
+            "COREDUMP": b"(\xb5/\xfd$\x0ca\x00\x00mocked core\nG\xfe\xe0\x10",
+            "COREDUMP_CMDLINE": "python3",
+            "COREDUMP_CWD": "/",
+            "COREDUMP_ENVIRON": "SHELL=/bin/bash\n",
+            "COREDUMP_EXE": sys.executable,
+            "COREDUMP_GID": 0,
+            "COREDUMP_PID": 123456789,
+            "COREDUMP_PROC_MAPS": "mocked /proc/<pid>/maps",
+            "COREDUMP_PROC_STATUS": "mocked /proc/<pid>/status",
+            "COREDUMP_SIGNAL": 11,
+            "COREDUMP_SIGNAL_NAME": "SIGSEGV",
+            "COREDUMP_TIMESTAMP": datetime.datetime(
+                2024, 2, 19, 12, 10, 42, tzinfo=datetime.timezone.utc
+            ),
+            "COREDUMP_UID": 0,
+        }
+        process_crash_mock.side_effect = lambda report, *_args: report
+
+        report = apport_binary.process_crash_from_systemd_coredump("3-12345-7")
+
+        get_systemd_coredump_mock.assert_called_once_with("3-12345-7")
+        process_crash_mock.assert_called_once()
+        self.assertEqual(report.pid, 123456789)
+        self.assertEqual(
+            dict(report),
+            {
+                "Date": "Mon Feb 19 12:10:42 2024",
+                "CoreDump": b"(\xb5/\xfd$\x0ca\x00\x00mocked core\nG\xfe\xe0\x10",
+                "ExecutablePath": sys.executable,
+                "ExecutableTimestamp": str(int(os.stat(sys.executable).st_mtime)),
+                "ProblemType": "Crash",
+                "ProcCmdline": "python3",
+                "ProcCwd": "/",
+                "ProcEnviron": "SHELL=/bin/bash",
+                "ProcMaps": "mocked /proc/<pid>/maps",
+                "ProcStatus": "mocked /proc/<pid>/status",
+                "Signal": "11",
+                "SignalName": "SIGSEGV",
+            },
+        )
